"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _pathHelpers = require("../helpers/pathHelpers");

var _CheckboxInput = _interopRequireDefault(require("../../styles/cspace-input/CheckboxInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = {
  asText: _propTypes.default.bool,
  className: _propTypes.default.string,
  embedded: _propTypes.default.bool,

  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  readOnly: _propTypes.default.bool,
  transition: _propTypes.default.objectOf(_propTypes.default.bool),
  trueLabel: _propTypes.default.string,
  falseLabel: _propTypes.default.string,
  indeterminateLabel: _propTypes.default.string,
  value: _propTypes.default.oneOfType([_propTypes.default.bool, _propTypes.default.string, _propTypes.default.number]),
  onCommit: _propTypes.default.func,
  onClick: _propTypes.default.func
};
const defaultProps = {
  asText: undefined,
  className: undefined,
  embedded: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  readOnly: undefined,
  transition: {
    null: true,
    true: false,
    false: true
  },
  trueLabel: 'yes',
  falseLabel: 'no',
  indeterminateLabel: 'indeterminate',
  value: undefined,
  onCommit: undefined,
  onClick: undefined
};

class CheckboxInput extends _react.Component {
  constructor() {
    super();
    this.handleChange = this.handleChange.bind(this);
  }

  handleChange() {
    const {
      transition,
      value,
      onCommit
    } = this.props;

    if (onCommit) {
      const normalizedValue = typeof value === 'undefined' ? null : value;
      const newValue = transition[normalizedValue];
      onCommit((0, _pathHelpers.getPath)(this.props), newValue);
    }
  }

  render() {
    const {
      asText,
      className,
      embedded,
      name,
      parentPath,
      readOnly,
      subpath,
      transition,
      value,
      trueLabel,
      falseLabel,
      indeterminateLabel,
      onClick,
      onCommit,
      ...remainingProps
    } = this.props;
    let checked;
    let textValue;
    let classes = (0, _classnames.default)({
      [_CheckboxInput.default.readOnly]: readOnly,
      [_CheckboxInput.default.normal]: !readOnly,
      [_CheckboxInput.default.embedded]: embedded
    }, className);

    if (value === true) {
      checked = true;
      textValue = trueLabel;
    } else if (value === false) {
      checked = false;
      textValue = falseLabel;
    } else {
      checked = false;
      textValue = indeterminateLabel;
      classes = (0, _classnames.default)(classes, _CheckboxInput.default.indeterminate);
    }

    if (asText) {
      return /*#__PURE__*/_react.default.createElement("div", {
        className: classes
      }, textValue);
    } // FIXME: Don't break jsx-a11y/label-has-associated-control.

    /* eslint-disable
       jsx-a11y/label-has-associated-control,
       jsx-a11y/no-noninteractive-element-interactions,
       jsx-a11y/click-events-have-key-events */


    return /*#__PURE__*/_react.default.createElement("label", {
      className: classes,
      onClick: onClick
    }, /*#__PURE__*/_react.default.createElement("input", _extends({
      checked: checked,
      "data-name": name,
      disabled: readOnly,
      type: "checkbox",
      onChange: this.handleChange // eslint-disable-next-line react/jsx-props-no-spreading

    }, remainingProps)), /*#__PURE__*/_react.default.createElement("span", null));
    /* eslint-enable
       jsx-a11y/label-has-associated-control,
       jsx-a11y/no-noninteractive-element-interactions,
       jsx-a11y/click-events-have-key-events */
  }

}

exports.default = CheckboxInput;
CheckboxInput.propTypes = propTypes;
CheckboxInput.defaultProps = defaultProps;