"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _pathHelpers = require("../helpers/pathHelpers");

var _ChooserInput = _interopRequireDefault(require("../../styles/cspace-input/ChooserInput.css"));

var _MiniButton = _interopRequireDefault(require("../../styles/cspace-input/MiniButton.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = {
  children: _propTypes.default.node,
  chooseButtonLabel: _propTypes.default.string,
  className: _propTypes.default.string,
  // TODO: Stop using propTypes in isInput. Until then, these unused props need to be declared so
  // this component is recognized as an input.

  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  // eslint-disable-next-line react/forbid-prop-types
  value: _propTypes.default.any,
  readOnly: _propTypes.default.bool,
  formatValue: _propTypes.default.func,
  onChooseButtonClick: _propTypes.default.func,
  onDrop: _propTypes.default.func
};
const defaultProps = {
  children: undefined,
  chooseButtonLabel: 'Select…',
  className: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  formatValue: value => value,
  onChooseButtonClick: undefined,
  onDrop: undefined
};

class ChooserInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleDragEnter = this.handleDragEnter.bind(this);
    this.handleDragLeave = this.handleDragLeave.bind(this);
    this.handleDragOver = this.handleDragOver.bind(this);
    this.handleDrop = this.handleDrop.bind(this);
    this.state = {
      isDraggedOver: false
    };
  }

  handleDragEnter(event) {
    event.preventDefault();
    event.stopPropagation();
    this.setState({
      isDraggedOver: true
    });
  }

  handleDragLeave(event) {
    event.preventDefault();
    event.stopPropagation();
    this.setState({
      isDraggedOver: false
    });
  } // eslint-disable-next-line class-methods-use-this


  handleDragOver(event) {
    event.preventDefault();
    event.stopPropagation();
  }

  handleDrop(event) {
    const {
      onDrop
    } = this.props;

    if (onDrop) {
      event.preventDefault();
      event.stopPropagation();
      onDrop(event.dataTransfer);
      this.setState({
        isDraggedOver: false
      });
    }
  }

  render() {
    const {
      children,
      chooseButtonLabel,
      className,
      value,
      readOnly,
      formatValue,
      onChooseButtonClick,
      onDrop
    } = this.props;
    const {
      isDraggedOver
    } = this.state;
    const formattedValue = formatValue(value);
    const classes = (0, _classnames.default)(className, isDraggedOver ? _ChooserInput.default.dragOver : _ChooserInput.default.normal);
    let dragDropProps;

    if (onDrop && !readOnly) {
      dragDropProps = {
        onDragEnter: this.handleDragEnter,
        onDragLeave: this.handleDragLeave,
        onDragOver: this.handleDragOver,
        onDrop: this.handleDrop
      };
    }

    let chooseButton;

    if (!readOnly) {
      chooseButton = /*#__PURE__*/_react.default.createElement("button", {
        className: _MiniButton.default.common,
        disabled: readOnly,
        name: "choose",
        type: "button",
        onClick: onChooseButtonClick
      }, chooseButtonLabel);
    }

    return (
      /*#__PURE__*/
      // Allow click/drag/drop events on the file info div.
      _react.default.createElement("div", {
        className: classes
      }, children, chooseButton, /*#__PURE__*/_react.default.createElement("div", _extends({
        onClick: readOnly ? undefined : onChooseButtonClick // eslint-disable-next-line react/jsx-props-no-spreading

      }, dragDropProps), formattedValue))
    );
  }

}

exports.default = ChooserInput;
ChooserInput.propTypes = propTypes;
ChooserInput.defaultProps = defaultProps;