"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _DropdownMenuInput = _interopRequireDefault(require("./DropdownMenuInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _optionHelpers = require("../helpers/optionHelpers");

var _pathHelpers = require("../helpers/pathHelpers");

var _ComboBoxInput = _interopRequireDefault(require("../../styles/cspace-input/ComboBoxInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const DropdownMenuInput = (0, _changeable.default)(_DropdownMenuInput.default);
const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ...DropdownMenuInput.propTypes,
  blankable: _propTypes.default.bool,
  onAddOption: _propTypes.default.func
};
const defaultProps = {
  blankable: true,
  onAddOption: undefined
};

class ComboBoxInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleDropdownInputChange = this.handleDropdownInputChange.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputCommit = this.handleDropdownInputCommit.bind(this);
    this.handleDropdownInputKeyDown = this.handleDropdownInputKeyDown.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    const {
      value
    } = this.props;
    this.state = {
      value,
      isAdding: false,
      open: false
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  commit(value) {
    const {
      onCommit
    } = this.props;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  handleDropdownInputChange(value) {
    this.setState({
      isAdding: true,
      open: true,
      valueLabel: value
    });
  }

  handleDropdownInputCommit(path, value) {
    this.setState({
      value,
      isAdding: false,
      open: false
    });
    this.commit(value);
  }

  handleDropdownInputKeyDown(event) {
    if (event.key === 'Enter') {
      const {
        isAdding,
        valueLabel
      } = this.state;

      if (isAdding) {
        const {
          blankable,
          options,
          onAddOption
        } = this.props; // If the current content of the input matches an option exactly, select it.

        const matchingOption = (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

        if (matchingOption) {
          this.setState({
            isAdding: false,
            open: false,
            value: matchingOption.value,
            valueLabel: matchingOption.valueLabel
          });
          this.commit(matchingOption.value);
        } else if (valueLabel !== '' || blankable) {
          this.setState({
            valueLabel,
            open: false
          });

          if (onAddOption) {
            onAddOption(valueLabel);
          }
        }
      }
    }
  }

  handleDropdownInputClose() {
    this.setState({
      isAdding: false,
      open: false
    });
    const {
      onClose
    } = this.props;

    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });
  }

  render() {
    const {
      isAdding,
      open,
      value,
      valueLabel
    } = this.state;
    const {
      className,
      onAddOption,
      ...remainingProps
    } = this.props;
    const {
      options
    } = remainingProps;
    let valueProp;

    if (isAdding) {
      valueProp = {
        valueLabel
      };
    } else if ((0, _optionHelpers.getOptionForValue)(options, value)) {
      valueProp = {
        value
      };
    } else {
      valueProp = {
        valueLabel: value
      };
    }

    const classes = (0, _classnames.default)(className, {
      [_ComboBoxInput.default.adding]: isAdding
    });
    return /*#__PURE__*/_react.default.createElement(DropdownMenuInput // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, valueProp, {
      className: classes,
      open: open,
      onChange: this.handleDropdownInputChange,
      onClose: this.handleDropdownInputClose,
      onCommit: this.handleDropdownInputCommit,
      onKeyDown: this.handleDropdownInputKeyDown,
      onOpen: this.handleDropdownInputOpen
    }));
  }

}

exports.default = ComboBoxInput;
ComboBoxInput.propTypes = propTypes;
ComboBoxInput.defaultProps = defaultProps;