"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _immutable = _interopRequireDefault(require("immutable"));

var _classnames = _interopRequireDefault(require("classnames"));

var _get = _interopRequireDefault(require("lodash/get"));

var _pathHelpers = require("../helpers/pathHelpers");

var _inputHelpers = require("../helpers/inputHelpers");

var _CompoundInput = _interopRequireDefault(require("../../styles/cspace-input/CompoundInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const propTypes = {
  children: _propTypes.default.node,
  className: _propTypes.default.string,
  defaultChildSubpath: _pathHelpers.pathPropType,
  name: _propTypes.default.string,
  // TODO: Stop using propTypes in isInput. Until then, these unused props need to be declared so
  // this component is recognized as an input.

  /* eslint-disable react/no-unused-prop-types */
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  readOnly: _propTypes.default.bool,
  value: _propTypes.default.oneOfType([_propTypes.default.object, _propTypes.default.instanceOf(_immutable.default.Map)])
};
const defaultProps = {
  children: undefined,
  className: undefined,
  defaultChildSubpath: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  readOnly: undefined,
  value: {}
};

const getChildValue = (value, subpath, name) => {
  let keyPath = [];

  if (subpath) {
    keyPath = keyPath.concat(subpath);
  }

  if (name) {
    keyPath = keyPath.concat(name);
  }

  if (keyPath.length === 0) {
    return value;
  }

  if (_immutable.default.Map.isMap(value)) {
    return value.getIn(keyPath);
  }

  return (0, _get.default)(value, keyPath);
};

class CustomCompoundInput extends _react.Component {
  decorateInputs(children) {
    const {
      readOnly
    } = this.props;
    return _react.default.Children.map(children, child => {
      if (!child || !child.type) {
        // Undefined/null child or text node. Just return it.
        return child;
      }

      if ((0, _inputHelpers.isInput)(child)) {
        const {
          name
        } = child.props;
        let {
          subpath
        } = child.props;
        const {
          defaultChildSubpath,
          value
        } = this.props;

        if (typeof subpath === 'undefined') {
          subpath = defaultChildSubpath;
        }

        return /*#__PURE__*/_react.default.cloneElement(child, {
          readOnly,
          subpath,
          parentPath: (0, _pathHelpers.getPath)(this.props),
          value: getChildValue(value, subpath, name)
        });
      }

      return /*#__PURE__*/_react.default.cloneElement(child, {
        children: this.decorateInputs(child.props.children)
      });
    }, this);
  }

  render() {
    const {
      children,
      className,
      name,
      readOnly
    } = this.props;
    const classes = (0, _classnames.default)(className, _CompoundInput.default.common, {
      [_CompoundInput.default.readOnly]: readOnly
    });
    return /*#__PURE__*/_react.default.createElement("fieldset", {
      className: classes,
      "data-name": name
    }, this.decorateInputs(children));
  }

}

exports.default = CustomCompoundInput;
CustomCompoundInput.propTypes = propTypes;
CustomCompoundInput.defaultProps = defaultProps;