"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _reactCalendar = _interopRequireDefault(require("react-calendar"));

var _DropdownInput = _interopRequireDefault(require("./DropdownInput"));

var _LineInput = _interopRequireDefault(require("./LineInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _committable = _interopRequireDefault(require("../enhancers/committable"));

var _pathHelpers = require("../helpers/pathHelpers");

var _DateInput = _interopRequireDefault(require("../../styles/cspace-input/DateInput.css"));

require("!style-loader!css-loader!../../styles/react-calendar/calendar.css");

var _dateHelpers = require("../helpers/dateHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const DropdownInput = (0, _committable.default)((0, _changeable.default)(_DropdownInput.default));
const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ..._DropdownInput.default.propTypes,
  locale: _propTypes.default.string,
  onCommit: _propTypes.default.func,
  readOnly: _propTypes.default.bool
};
const defaultProps = {
  locale: 'en-US',
  onCommit: undefined,
  readOnly: undefined
};

class DateInput extends _react.Component {
  constructor(props) {
    super(props);
    this.focusCalendar = this.focusCalendar.bind(this);
    this.handleCalendarChange = this.handleCalendarChange.bind(this);
    this.handleCalendarContainerRef = this.handleCalendarContainerRef.bind(this); // this.handleCalendarRef = this.handleCalendarRef.bind(this);

    this.handleDropdownInputApi = this.handleDropdownInputApi.bind(this);
    this.handleDropdownInputBeforeClose = this.handleDropdownInputBeforeClose.bind(this);
    this.handleDropdownInputChange = this.handleDropdownInputChange.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputKeyDown = this.handleDropdownInputKeyDown.bind(this);
    this.handleDropdownInputMount = this.handleDropdownInputMount.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    const value = (0, _dateHelpers.normalizeISO8601DateString)(props.value) || props.value;
    this.state = {
      value,
      date: (0, _dateHelpers.parseNormalizedDate)(value),
      open: false
    };
  }

  componentWillReceiveProps(nextProps) {
    const nextValue = (0, _dateHelpers.normalizeISO8601DateString)(nextProps.value) || nextProps.value;
    this.setState({
      value: nextValue,
      date: (0, _dateHelpers.parseNormalizedDate)(nextValue)
    });
  }

  commit(date) {
    const {
      onCommit
    } = this.props;
    const {
      value: initialValue
    } = this.props;
    const normalizedInitialValue = (0, _dateHelpers.normalizeISO8601DateString)(initialValue);
    const nextValue = (0, _dateHelpers.formatDate)(date);

    if (onCommit && (nextValue || normalizedInitialValue) && nextValue !== normalizedInitialValue) {
      onCommit((0, _pathHelpers.getPath)(this.props), nextValue);
    }
  }

  focusCalendar() {
    // TODO: react-calendar v6 no longer has the focus method. Maybe it will come back?
    // if (this.calendar) {
    //   this.calendar.focus();
    // }
    // For now do a bit of a DOM hack to focus.
    if (this.calendarContainerDomNode) {
      const button = this.calendarContainerDomNode.querySelector('button');

      if (button) {
        button.focus();
      }
    }
  }

  handleCalendarChange(date) {
    this.setState({
      value: (0, _dateHelpers.formatDate)(date),
      open: false
    });
    this.commit(date);
    this.focusInput();
  } // handleCalendarRef(ref) {
  //   this.calendar = ref;
  // }


  handleCalendarContainerRef(ref) {
    this.calendarContainerDomNode = ref;
  }

  handleDropdownInputApi(api) {
    this.dropdownInputApi = api;
  }

  handleDropdownInputBeforeClose(isCancelled) {
    if (isCancelled) {
      const {
        value
      } = this.props;
      this.setState({
        provisionalDate: undefined,
        value: (0, _dateHelpers.normalizeISO8601DateString)(value)
      });
    }
  }

  handleDropdownInputChange(value) {
    const {
      locale
    } = this.props;
    const date = (0, _dateHelpers.parseNormalizedDate)((0, _dateHelpers.normalizeDateString)(value, locale));
    this.setState({
      value,
      provisionalDate: date,
      open: true
    });
  }

  handleDropdownInputClose() {
    const nextState = {
      provisionalDate: undefined,
      open: false
    };
    const {
      provisionalDate,
      value
    } = this.state;

    if (typeof provisionalDate !== 'undefined' && value === '') {
      // Normally enter must be pressed on the provisional value in order to select a matching
      // date, but make an exception for the blank value. This allows fields to be cleared
      // without ever pressing enter. This is required by DRYD-227.
      nextState.value = '';
      const {
        onCommit
      } = this.props;

      if (onCommit) {
        onCommit((0, _pathHelpers.getPath)(this.props), '');
      }
    } else {
      const {
        value: origValue
      } = this.props;
      nextState.value = (0, _dateHelpers.normalizeISO8601DateString)(origValue) || origValue;
    }

    this.setState(nextState);
  }

  handleDropdownInputKeyDown(event) {
    const {
      provisionalDate,
      value
    } = this.state;

    if (event.key === 'Tab') {
      // DRYD-264: Close the dropdown when tab is depressed. The calendar is still keyboard
      // accessible by pressing down arrow. Need to close immediately (synchronously) -- otherwise
      // the calendar will receive focus before closing, leaving nothing focused. To do this, the
      // close() method of the dropdown needs to be called, rather than just passing open={false}
      // as a prop.
      this.dropdownInputApi.close();
      this.setState({
        open: false
      });
    } else if (typeof provisionalDate !== 'undefined' && event.key === 'Enter') {
      event.preventDefault();

      if (provisionalDate !== null || value === '') {
        this.setState({
          open: false
        });
        this.commit(provisionalDate);
      }
    }
  }

  handleDropdownInputMount({
    focusInput
  }) {
    this.focusInput = focusInput;
  }

  handleDropdownInputOpen() {
    const {
      open
    } = this.state;

    if (!open) {
      this.setState({
        open: true,
        provisionalDate: undefined
      });
    }
  }

  render() {
    const {
      locale,
      readOnly,
      ...remainingProps
    } = this.props;
    const {
      open,
      date,
      provisionalDate,
      value
    } = this.state;

    if (readOnly) {
      const {
        embedded
      } = remainingProps;
      return /*#__PURE__*/_react.default.createElement(_LineInput.default, {
        readOnly: true,
        value: value,
        embedded: embedded
      });
    }

    let calendarValue;

    if (value) {
      calendarValue = typeof provisionalDate !== 'undefined' ? provisionalDate : date;
    }

    const className = typeof provisionalDate !== 'undefined' ? _DateInput.default.provisional : _DateInput.default.normal;
    return /*#__PURE__*/_react.default.createElement(DropdownInput // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      className: className,
      focusPopup: this.focusCalendar,
      open: open,
      value: value,
      api: this.handleDropdownInputApi,
      onChange: this.handleDropdownInputChange,
      onBeforeClose: this.handleDropdownInputBeforeClose,
      onClose: this.handleDropdownInputClose,
      onKeyDown: this.handleDropdownInputKeyDown,
      onMount: this.handleDropdownInputMount,
      onOpen: this.handleDropdownInputOpen
    }), /*#__PURE__*/_react.default.createElement("div", {
      ref: this.handleCalendarContainerRef
    }, /*#__PURE__*/_react.default.createElement(_reactCalendar.default, {
      locale: locale // ref={this.handleCalendarRef}
      ,
      value: calendarValue,
      onChange: this.handleCalendarChange
    })));
  }

}

exports.default = DateInput;
DateInput.propTypes = propTypes;
DateInput.defaultProps = defaultProps;