"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _Popup = _interopRequireDefault(require("cspace-layout/lib/components/Popup"));

var _classnames = _interopRequireDefault(require("classnames"));

var _TextInput = _interopRequireDefault(require("./TextInput"));

var _DropdownInput = _interopRequireDefault(require("../../styles/cspace-input/DropdownInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ..._TextInput.default.propTypes,
  children: _propTypes.default.node,
  className: _propTypes.default.string,
  embedded: _propTypes.default.bool,
  open: _propTypes.default.bool,
  openOnFocus: _propTypes.default.bool,
  openOnMouseDown: _propTypes.default.bool,
  isOpenableWhenReadOnly: _propTypes.default.bool,
  api: _propTypes.default.func,

  /*
   * A function that may be called to give focus to the contents of the popup. This is supplied as
   * a prop because DropdownInput does not know the contents of the popup. Only the caller knows
   * which element in the popup should receive focus.
   */
  focusPopup: _propTypes.default.func,
  onBlur: _propTypes.default.func,
  onBeforeClose: _propTypes.default.func,
  onClose: _propTypes.default.func,
  onKeyDown: _propTypes.default.func,
  onMount: _propTypes.default.func,
  onOpen: _propTypes.default.func
};
const defaultProps = {
  children: undefined,
  className: undefined,
  embedded: undefined,
  open: undefined,
  openOnFocus: undefined,
  openOnMouseDown: true,
  isOpenableWhenReadOnly: undefined,
  api: undefined,
  focusPopup: undefined,
  onBlur: undefined,
  onBeforeClose: undefined,
  onClose: undefined,
  onKeyDown: undefined,
  onMount: undefined,
  onOpen: undefined
};

class DropdownInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleInputBlur = this.handleInputBlur.bind(this);
    this.handleInputFocus = this.handleInputFocus.bind(this);
    this.handleInputMouseDown = this.handleInputMouseDown.bind(this);
    this.handleInputKeyDown = this.handleInputKeyDown.bind(this);
    this.handlePopupBlur = this.handlePopupBlur.bind(this);
    this.handlePopupKeyDown = this.handlePopupKeyDown.bind(this);
    this.handlePopupMount = this.handlePopupMount.bind(this);
    this.handleRef = this.handleRef.bind(this);
    this.state = {
      open: props.open
    };
  }

  componentDidMount() {
    const {
      api,
      onMount
    } = this.props;

    if (api) {
      api({
        close: this.close.bind(this)
      });
    }

    if (onMount) {
      onMount({
        focusInput: this.focusInput.bind(this)
      });
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.open) {
      this.open();
    } else {
      this.asyncClose();
    }
  }

  componentDidUpdate(prevProps, prevState) {
    const {
      open: prevOpen
    } = prevState;
    const {
      open
    } = this.state;

    if (prevOpen && !open) {
      const {
        onClose
      } = this.props;

      if (onClose) {
        onClose();
      }
    } else if (!prevOpen && open) {
      const {
        onOpen
      } = this.props;

      if (onOpen) {
        onOpen();
      }
    }
  }

  close(isCancelled) {
    const {
      open
    } = this.state;

    if (open) {
      const {
        onBeforeClose
      } = this.props;

      if (onBeforeClose) {
        onBeforeClose(isCancelled);
      }

      this.setState({
        open: false
      });
    }
  }

  asyncClose(isCancelled) {
    setTimeout(() => {
      this.close(isCancelled);
    }, 0);
  }

  open() {
    const {
      open
    } = this.state;

    if (!open) {
      this.setState({
        open: true
      });
    }
  }

  focusInput() {
    // FIXME: This breaks the TextInput component abstraction by selecting the rendered DOM node
    // directly and calling focus() on it. But fixing this would require saving a ref to the
    // TextInput; making TextInput a class instead of a function; and adding a focus() method to
    // that class, which could be called here. LineInput and MultilineInput would also need to
    // become classes instead of functions, each with their own focus() method, in order to
    // properly implement TextInput.focus(). This seems like overkill at the moment.
    this.domNode.querySelector('input, textarea').focus();
  }

  focusPopup() {
    const {
      focusPopup
    } = this.props;

    if (focusPopup) {
      focusPopup();
    }
  }

  handleInputBlur(event) {
    const {
      onBlur
    } = this.props;

    if (onBlur) {
      onBlur(event);
    }

    if (!this.domNode.contains(event.relatedTarget)) {
      this.asyncClose();
    }
  }

  handleInputFocus() {
    const {
      openOnFocus
    } = this.props;

    if (openOnFocus) {
      this.open();
    }
  }

  handleInputMouseDown() {
    const {
      openOnMouseDown
    } = this.props;

    if (openOnMouseDown) {
      this.open();
    }
  }

  handleInputKeyDown(event) {
    const {
      onKeyDown
    } = this.props;

    if (event.key === 'ArrowDown') {
      const {
        open
      } = this.state; // Prevent the page from scrolling.

      event.preventDefault();

      if (open) {
        this.focusPopup();
      } else {
        this.open();
        this.focusPopupNeeded = true;
      }
    } else if (event.key === 'Escape') {
      this.asyncClose(true);
    }

    if (onKeyDown) {
      onKeyDown(event);
    }
  }

  handlePopupBlur(event) {
    if (!this.domNode.contains(event.relatedTarget)) {
      this.asyncClose();
    }
  }

  handlePopupKeyDown(event) {
    if (event.key === 'Escape') {
      this.asyncClose();
      this.focusInput();
    }
  }

  handlePopupMount() {
    if (this.focusPopupNeeded) {
      this.focusPopup();
      this.focusPopupNeeded = false;
    }
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  renderInput() {
    const {
      children,
      className,
      focusPopup,
      isOpenableWhenReadOnly,
      openOnFocus,
      openOnMouseDown,
      onBeforeClose,
      onClose,
      onKeyDown,
      onMount,
      onOpen,
      ...remainingProps
    } = this.props;
    return /*#__PURE__*/_react.default.createElement(_TextInput.default // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      onBlur: this.handleInputBlur,
      onFocus: this.handleInputFocus,
      onKeyDown: this.handleInputKeyDown,
      onMouseDown: this.handleInputMouseDown
    }));
  }

  renderDropdown() {
    const {
      open
    } = this.state;
    const {
      children
    } = this.props;

    if (open) {
      return /*#__PURE__*/_react.default.createElement(_Popup.default, {
        onBlur: this.handlePopupBlur,
        onKeyDown: this.handlePopupKeyDown,
        onMount: this.handlePopupMount
      }, children);
    }

    return null;
  }

  render() {
    const {
      open
    } = this.state;
    const {
      className,
      embedded,
      readOnly,
      isOpenableWhenReadOnly
    } = this.props;
    const classes = (0, _classnames.default)(className, {
      [_DropdownInput.default.normal]: !embedded,
      [_DropdownInput.default.embedded]: embedded,
      [_DropdownInput.default.open]: open
    });
    let readOnlyProps;

    if (readOnly && isOpenableWhenReadOnly) {
      readOnlyProps = {
        tabIndex: 0,
        onBlur: this.handleInputBlur,
        onFocus: this.handleInputFocus,
        onKeyDown: this.handleInputKeyDown,
        onMouseDown: this.handleInputMouseDown
      };
    }

    return /*#__PURE__*/_react.default.createElement("div", _extends({
      ref: this.handleRef,
      className: classes // eslint-disable-next-line react/jsx-props-no-spreading

    }, readOnlyProps), this.renderInput(), this.renderDropdown());
  }

}

exports.default = DropdownInput;
DropdownInput.propTypes = propTypes;
DropdownInput.defaultProps = defaultProps;