"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _LineInput = _interopRequireDefault(require("./LineInput"));

var _DropdownInput = _interopRequireDefault(require("./DropdownInput"));

var _Menu = _interopRequireDefault(require("./Menu"));

var _pathHelpers = require("../helpers/pathHelpers");

var _optionHelpers = require("../helpers/optionHelpers");

var _DropdownMenuInput = _interopRequireDefault(require("../../styles/cspace-input/DropdownMenuInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ..._DropdownInput.default.propTypes,
  blankable: _propTypes.default.bool,
  className: _propTypes.default.string,
  menuHeader: _propTypes.default.node,
  menuFooter: _propTypes.default.node,
  open: _propTypes.default.bool,
  options: _propTypes.default.arrayOf(_propTypes.default.shape({
    value: _propTypes.default.string,
    label: _propTypes.default.string
  })),
  ignoreDisabledOptions: _propTypes.default.bool,
  readOnly: _propTypes.default.bool,
  renderItemLabel: _propTypes.default.func,
  valueLabel: _propTypes.default.string,
  onClose: _propTypes.default.func,
  onCommit: _propTypes.default.func,
  onItemMouseEnter: _propTypes.default.func,
  onItemMouseLeave: _propTypes.default.func,
  onMount: _propTypes.default.func,
  onOpen: _propTypes.default.func,
  onUpdate: _propTypes.default.func
};
const defaultProps = {
  blankable: undefined,
  className: undefined,
  menuHeader: undefined,
  menuFooter: undefined,
  open: undefined,
  options: [],
  ignoreDisabledOptions: undefined,
  readOnly: undefined,
  renderItemLabel: undefined,
  valueLabel: undefined,
  onClose: undefined,
  onCommit: undefined,
  onItemMouseEnter: undefined,
  onItemMouseLeave: undefined,
  onMount: undefined,
  onOpen: undefined,
  onUpdate: undefined
};

const renderMenuHeader = content => {
  if (content) {
    return /*#__PURE__*/_react.default.createElement("header", null, content);
  }

  return null;
};

const renderMenuFooter = content => {
  if (content) {
    return /*#__PURE__*/_react.default.createElement("footer", null, content);
  }

  return null;
};

class DropdownMenuInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputMount = this.handleDropdownInputMount.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);
    this.focusMenu = this.focusMenu.bind(this);
    const valueLabel = props.valueLabel === null || typeof props.valueLabel === 'undefined' ? (0, _optionHelpers.getLabelForValue)(props.options, props.value) : props.valueLabel;
    this.state = {
      valueLabel,
      open: false,
      value: props.value
    };
  }

  componentDidMount() {
    const {
      onMount
    } = this.props;

    if (onMount) {
      const {
        value
      } = this.state;
      onMount({
        value,
        focusMenu: this.focusMenu.bind(this)
      });
    }
  }

  componentWillReceiveProps(nextProps) {
    const valueLabel = nextProps.valueLabel === null || typeof nextProps.valueLabel === 'undefined' ? (0, _optionHelpers.getLabelForValue)(nextProps.options, nextProps.value) : nextProps.valueLabel;
    this.setState({
      valueLabel,
      open: nextProps.open,
      value: nextProps.value
    });
  }

  componentDidUpdate() {
    const {
      onUpdate
    } = this.props;

    if (onUpdate) {
      const {
        value
      } = this.state;
      onUpdate({
        value
      });
    }
  }

  commit(value, meta) {
    const {
      value: initialValue,
      onCommit
    } = this.props;

    if (onCommit && (value || initialValue) && value !== initialValue) {
      onCommit((0, _pathHelpers.getPath)(this.props), value, meta);
    }
  }

  focusMenu(itemIndex) {
    if (this.menu) {
      this.menu.focus(itemIndex);
    }
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });
    const {
      onClose
    } = this.props;

    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputMount({
    focusInput
  }) {
    this.focusInput = focusInput;
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });
    const {
      onOpen
    } = this.props;

    if (onOpen) {
      onOpen();
    }
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(option) {
    const {
      value,
      meta,
      label: valueLabel
    } = option;
    this.setState({
      value,
      valueLabel,
      open: false
    });
    this.commit(value, meta);
    this.focusInput();
  }

  render() {
    const {
      open,
      value,
      valueLabel
    } = this.state;
    const {
      blankable,
      className,
      focusPopup,
      ignoreDisabledOptions,
      menuHeader,
      menuFooter,
      onBeforeItemFocusChange,
      onClose,
      onCommit,
      onItemMouseEnter,
      onItemMouseLeave,
      onMount,
      onOpen,
      onUpdate,
      open: openProp,
      options,
      readOnly,
      renderItemLabel,
      valueLabel: valueLabelProp,
      ...remainingProps
    } = this.props;
    const inputValue = valueLabel;

    if (readOnly) {
      const {
        embedded
      } = remainingProps;
      return /*#__PURE__*/_react.default.createElement(_LineInput.default, {
        readOnly: true,
        value: inputValue,
        embedded: embedded
      });
    }

    const classes = (0, _classnames.default)(className, {
      [_DropdownMenuInput.default.common]: true,
      [_DropdownMenuInput.default.open]: open
    });
    return /*#__PURE__*/_react.default.createElement(_DropdownInput.default // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      className: classes,
      open: open,
      spellCheck: false,
      value: inputValue,
      onClose: this.handleDropdownInputClose,
      onMount: this.handleDropdownInputMount,
      onOpen: this.handleDropdownInputOpen,
      focusPopup: focusPopup || this.focusMenu
    }), renderMenuHeader(menuHeader), /*#__PURE__*/_react.default.createElement(_Menu.default, {
      options: options,
      ignoreDisabledOptions: ignoreDisabledOptions,
      ref: this.handleMenuRef,
      tabIndex: "-1",
      renderItemLabel: renderItemLabel,
      value: value,
      onSelect: this.handleMenuSelect,
      onBeforeItemFocusChange: onBeforeItemFocusChange,
      onItemMouseEnter: onItemMouseEnter,
      onItemMouseLeave: onItemMouseLeave
    }), renderMenuFooter(menuFooter));
  }

}

exports.default = DropdownMenuInput;
DropdownMenuInput.propTypes = propTypes;
DropdownMenuInput.defaultProps = defaultProps;