"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _DropdownMenuInput = _interopRequireDefault(require("./DropdownMenuInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _optionHelpers = require("../helpers/optionHelpers");

var _pathHelpers = require("../helpers/pathHelpers");

var _FilteringDropdownMenuInput = _interopRequireDefault(require("../../styles/cspace-input/FilteringDropdownMenuInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const DropdownMenuInput = (0, _changeable.default)(_DropdownMenuInput.default);
const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ...DropdownMenuInput.propTypes,
  blankable: _propTypes.default.bool,
  formatStatusMessage: _propTypes.default.func,
  filter: _propTypes.default.func,
  onClose: _propTypes.default.func,
  onOpen: _propTypes.default.func,
  onCommit: _propTypes.default.func
};
const defaultProps = {
  blankable: true,
  formatStatusMessage: count => {
    const matches = count === 1 ? 'match' : 'matches';
    const num = count === 0 ? 'No' : count;
    return `${num} ${matches} found`;
  },
  filter: undefined,
  onClose: undefined,
  onOpen: undefined,
  onCommit: undefined
};

class FilteringDropdownMenuInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleDropdownInputBeforeClose = this.handleDropdownInputBeforeClose.bind(this);
    this.handleDropdownInputChange = this.handleDropdownInputChange.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputCommit = this.handleDropdownInputCommit.bind(this);
    this.handleDropdownInputKeyDown = this.handleDropdownInputKeyDown.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    const {
      value
    } = this.props;
    this.state = {
      value,
      isFiltering: false,
      open: false
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  close() {
    this.setState({
      open: false
    });
  }

  commit(value, meta) {
    const {
      onCommit
    } = this.props;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value, meta);
    }
  }

  filter(filterByValue) {
    const {
      filter
    } = this.props;

    if (filter) {
      filter(filterByValue);
    }
  }

  handleDropdownInputBeforeClose(isCancelled) {
    if (isCancelled) {
      this.setState({
        isFiltering: false
      });
    }
  }

  handleDropdownInputChange(value) {
    this.setState({
      isFiltering: true,
      open: true,
      valueLabel: value
    });
    this.filter(value);
  }

  handleDropdownInputCommit(path, value, meta) {
    this.setState({
      value,
      isFiltering: false,
      open: false
    });
    this.filter();
    this.commit(value, meta);
  }

  handleDropdownInputKeyDown(event) {
    const {
      isFiltering
    } = this.state;

    if (isFiltering && event.key === 'Enter') {
      event.preventDefault();
      const {
        valueLabel
      } = this.state;
      const {
        blankable,
        options,
        ignoreDisabledOptions
      } = this.props; // If there is only one option, select it. Otherwise, if the current content of the
      // input matches an option exactly, select it.

      let matchingOption = options.length === 1 ? options[0] : (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

      if (!matchingOption && blankable && valueLabel === '') {
        // If blankable, and the current content of the input is empty, allow it to be
        // committed even if there is no empty option.
        matchingOption = {
          value: '',
          valueLabel: ''
        };
      } // The matching option must not be disabled.


      if (matchingOption && matchingOption.disabled && !ignoreDisabledOptions) {
        matchingOption = null;
      }

      if (matchingOption) {
        this.setState({
          open: false,
          value: matchingOption.value,
          valueLabel: matchingOption.label
        });
        this.commit(matchingOption.value, matchingOption.meta);
      }
    }
  }

  handleDropdownInputClose() {
    const nextState = {
      isFiltering: false,
      open: false
    };
    const {
      isFiltering,
      valueLabel
    } = this.state;

    if (isFiltering && valueLabel === '') {
      // Normally enter must be pressed on the value being filtered in order to select a matching
      // option, but make an exception for the blank value. If the dropdown is closed while
      // filtering, and the filter value is blank, automatically select the corresponding option
      // if there is one, or if blankable is true. This allows fields to be cleared without ever
      // pressing enter. This is required by DRYD-227.
      const {
        blankable,
        options
      } = this.props;
      let matchingOption = (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

      if (!matchingOption && blankable) {
        // If blankable, and the current content of the input is empty, allow it to be
        // committed even if there is no empty option.
        matchingOption = {
          value: '',
          valueLabel: ''
        };
      }

      if (matchingOption) {
        nextState.value = matchingOption.value;
        nextState.valueLabel = matchingOption.label;
        this.commit(matchingOption.value, matchingOption.meta);
      }
    }

    this.setState(nextState);
    const {
      onClose
    } = this.props;

    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputOpen() {
    const {
      onOpen
    } = this.props;
    const {
      open
    } = this.state;

    if (onOpen) {
      onOpen();
    }

    if (!open) {
      this.setState({
        isFiltering: false,
        open: true
      });
      this.filter();
    }
  }

  renderMenuHeader() {
    const {
      isFiltering
    } = this.state;
    const {
      formatStatusMessage,
      options
    } = this.props;

    if (isFiltering) {
      return /*#__PURE__*/_react.default.createElement("span", null, formatStatusMessage(options ? options.length : 0));
    }

    return null;
  }

  render() {
    const {
      isFiltering,
      open,
      value,
      valueLabel
    } = this.state;
    const {
      className,
      formatStatusMessage,
      onMount,
      ...remainingProps
    } = this.props;
    const valueProp = isFiltering ? {
      valueLabel
    } : {
      value
    };
    const classes = (0, _classnames.default)(className, {
      [_FilteringDropdownMenuInput.default.filtering]: isFiltering
    });
    return /*#__PURE__*/_react.default.createElement(DropdownMenuInput // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, valueProp, {
      className: classes,
      menuHeader: this.renderMenuHeader(),
      open: open,
      onBeforeClose: this.handleDropdownInputBeforeClose,
      onChange: this.handleDropdownInputChange,
      onClose: this.handleDropdownInputClose,
      onCommit: this.handleDropdownInputCommit,
      onKeyDown: this.handleDropdownInputKeyDown,
      onOpen: this.handleDropdownInputOpen,
      onMount: onMount
    }));
  }

}

exports.default = FilteringDropdownMenuInput;
FilteringDropdownMenuInput.propTypes = propTypes;
FilteringDropdownMenuInput.defaultProps = defaultProps;