"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _Row = _interopRequireDefault(require("cspace-layout/lib/components/Row"));

var _DropdownInput = _interopRequireDefault(require("./DropdownInput"));

var _LineInput = _interopRequireDefault(require("./LineInput"));

var _Menu = _interopRequireDefault(require("./Menu"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _committable = _interopRequireDefault(require("../enhancers/committable"));

var _pathHelpers = require("../helpers/pathHelpers");

var _IDGeneratorInput = _interopRequireDefault(require("../../styles/cspace-input/IDGeneratorInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const DropdownInput = (0, _committable.default)((0, _changeable.default)(_DropdownInput.default));
const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
  // Until then, propTypes need to be hoisted from the base component.
  // eslint-disable-next-line react/forbid-foreign-prop-types
  ...DropdownInput.propTypes,
  patterns: _propTypes.default.arrayOf(_propTypes.default.shape({
    name: _propTypes.default.string,
    type: _propTypes.default.string,
    sample: _propTypes.default.string
  })),
  generateID: _propTypes.default.func,
  sampleColumnLabel: _propTypes.default.string,
  typeColumnLabel: _propTypes.default.string,
  readOnly: _propTypes.default.bool,
  onMount: _propTypes.default.func,
  onOpen: _propTypes.default.func
};
const defaultProps = {
  patterns: [],
  generateID: undefined,
  sampleColumnLabel: 'Sample',
  typeColumnLabel: 'Type',
  readOnly: undefined,
  onMount: undefined,
  onOpen: undefined
};

const renderItemLabel = label => /*#__PURE__*/_react.default.createElement(_Row.default, null, /*#__PURE__*/_react.default.createElement("div", null, label[0]), /*#__PURE__*/_react.default.createElement("div", null, label[1]));

class IDGeneratorInput extends _react.Component {
  constructor(props) {
    super(props);
    this.focusMenu = this.focusMenu.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputMount = this.handleDropdownInputMount.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);
    this.state = {
      open: false
    };
  }

  componentDidMount() {
    const {
      onMount
    } = this.props;

    if (onMount) {
      onMount();
    }
  }

  focusMenu() {
    if (this.menu) {
      this.menu.focus();
    }
  }

  handleDropdownInputMount({
    focusInput
  }) {
    this.focusInput = focusInput;
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });
  }

  handleDropdownInputOpen() {
    const {
      onOpen,
      patterns
    } = this.props;

    if (onOpen) {
      onOpen(patterns);
    }

    this.setState({
      open: true
    });
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(option) {
    this.setState({
      open: false
    });
    this.focusInput();
    const {
      generateID
    } = this.props;

    if (generateID) {
      generateID(option.value, (0, _pathHelpers.getPath)(this.props));
    }
  }

  render() {
    const {
      open
    } = this.state;
    const {
      generateID,
      onMount,
      onOpen,
      patterns,
      sampleColumnLabel,
      typeColumnLabel,
      readOnly,
      ...remainingProps
    } = this.props;
    const {
      embedded,
      value
    } = remainingProps;

    if (readOnly) {
      return /*#__PURE__*/_react.default.createElement(_LineInput.default, {
        embedded: embedded,
        readOnly: true,
        value: value
      });
    }

    const options = patterns.map(pattern => ({
      value: pattern.name,
      label: [pattern.type, pattern.sample]
    }));
    return /*#__PURE__*/_react.default.createElement(DropdownInput // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      className: _IDGeneratorInput.default.normal,
      focusPopup: this.focusMenu,
      open: open,
      onClose: this.handleDropdownInputClose,
      onMount: this.handleDropdownInputMount,
      onOpen: this.handleDropdownInputOpen
    }), /*#__PURE__*/_react.default.createElement("header", null, /*#__PURE__*/_react.default.createElement(_Row.default, null, /*#__PURE__*/_react.default.createElement("div", null, typeColumnLabel), /*#__PURE__*/_react.default.createElement("div", null, sampleColumnLabel))), /*#__PURE__*/_react.default.createElement(_Menu.default, {
      options: options,
      ref: this.handleMenuRef,
      renderItemLabel: renderItemLabel,
      tabIndex: "-1",
      onSelect: this.handleMenuSelect
    }));
  }

}

exports.default = IDGeneratorInput;
IDGeneratorInput.propTypes = propTypes;
IDGeneratorInput.defaultProps = defaultProps;