"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = InputTableHeader;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _inputHelpers = require("../helpers/inputHelpers");

var _normalizeLabel = _interopRequireDefault(require("../helpers/normalizeLabel"));

var _InputTableHeader = _interopRequireDefault(require("../../styles/cspace-input/InputTableHeader.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const propTypes = {
  children: _propTypes.default.node,
  embedded: _propTypes.default.bool,
  renderLabel: _propTypes.default.func,
  sortableFields: _propTypes.default.objectOf(_propTypes.default.bool),
  onSortButtonClick: _propTypes.default.func
};
const defaultProps = {
  children: undefined,
  embedded: undefined,
  renderLabel: input => (0, _normalizeLabel.default)(input.props.label),
  sortableFields: undefined,
  onSortButtonClick: undefined
};

function InputTableHeader(props) {
  const {
    children,
    embedded,
    renderLabel,
    sortableFields,
    onSortButtonClick
  } = props;
  const inputs = (0, _inputHelpers.extractInputs)(children);
  const labels = inputs.map(input => renderLabel(input));

  if (!labels.some(label => !!label)) {
    return null;
  }

  const labelContainers = labels.map((label, index) => {
    const input = inputs[index];
    const {
      flex,
      name
    } = input.props;
    const style = flex ? {
      flex
    } : undefined;
    let wrappedLabel = label;

    if (onSortButtonClick && sortableFields && sortableFields[name]) {
      wrappedLabel = /*#__PURE__*/_react.default.createElement("button", {
        "data-name": name,
        type: "button",
        onClick: onSortButtonClick
      }, label);
    }

    return /*#__PURE__*/_react.default.createElement("div", {
      key: name || index,
      style: style
    }, wrappedLabel);
  });
  const className = embedded ? _InputTableHeader.default.embedded : _InputTableHeader.default.normal;
  return /*#__PURE__*/_react.default.createElement("div", {
    className: className
  }, labelContainers);
}

InputTableHeader.propTypes = propTypes;
InputTableHeader.defaultProps = defaultProps;