"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _pathHelpers = require("../helpers/pathHelpers");

var _LineInput = _interopRequireDefault(require("../../styles/cspace-input/LineInput.css"));

var _LineInputWrapper = _interopRequireDefault(require("../../styles/cspace-input/LineInputWrapper.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = {
  asText: _propTypes.default.bool,
  'aria-label': _propTypes.default.string,
  autoComplete: _propTypes.default.string,
  embedded: _propTypes.default.bool,
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  value: _propTypes.default.string,
  readOnly: _propTypes.default.bool,
  showLink: _propTypes.default.bool,
  linkContent: _propTypes.default.oneOfType([_propTypes.default.element, _propTypes.default.string]),
  linkUrl: _propTypes.default.func,
  api: _propTypes.default.func
};
const defaultProps = {
  asText: undefined,
  'aria-label': undefined,
  autoComplete: undefined,
  embedded: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  showLink: undefined,
  linkContent: 'Open',
  linkUrl: value => {
    if (value && (value.startsWith('https://') || value.startsWith('http://'))) {
      return value;
    }

    return null;
  },
  api: undefined
};
/**
 * A text input that accepts and is able to display only a single line of text. If a value prop is
 * supplied that contains a newline character, the behavior is unspecified; newline characters may
 * be stripped, replaced with other characters, or retained but not displayed. If this presents a
 * problem, use TextInput or MultilineInput.
 */

class LineInput extends _react.Component {
  constructor() {
    super();
    this.handleRef = this.handleRef.bind(this);
  }

  componentDidMount() {
    const {
      api
    } = this.props;

    if (api) {
      api({
        focus: this.focus.bind(this)
      });
    }
  }

  componentWillUnmount() {
    const {
      api
    } = this.props;

    if (api) {
      api(null);
    }
  }

  focus() {
    this.domNode.focus();
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    const {
      api,
      asText,
      autoComplete,
      embedded,
      name,
      parentPath,
      readOnly,
      subpath,
      value,
      showLink,
      linkContent,
      linkUrl,
      ...remainingProps
    } = this.props;
    const className = embedded ? _LineInput.default.embedded : _LineInput.default.normal;
    const normalizedValue = value === null || typeof value === 'undefined' ? '' : value;

    if (asText) {
      return /*#__PURE__*/_react.default.createElement("div", {
        className: className
      }, normalizedValue);
    }

    const input = /*#__PURE__*/_react.default.createElement("input", _extends({
      // In Chrome, having autocomplete="off" on the parent form doesn't seem to be enough.
      autoComplete: autoComplete || 'off' // eslint-disable-next-line react/jsx-props-no-spreading

    }, remainingProps, {
      className: className,
      "data-name": name,
      disabled: readOnly,
      readOnly: !remainingProps.onChange,
      ref: this.handleRef,
      type: "text",
      value: normalizedValue
    }));

    if (showLink) {
      const url = linkUrl(value);
      const link = url ? /*#__PURE__*/_react.default.createElement("a", {
        href: url,
        target: "_blank",
        rel: "noopener noreferrer"
      }, linkContent) : null;
      return /*#__PURE__*/_react.default.createElement("div", {
        className: _LineInputWrapper.default.common
      }, input, link);
    }

    return input;
  }

}

exports.default = LineInput;
LineInput.propTypes = propTypes;
LineInput.defaultProps = defaultProps;