"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _pathHelpers = require("../helpers/pathHelpers");

var _MultilineInput = _interopRequireDefault(require("../../styles/cspace-input/MultilineInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = {
  asText: _propTypes.default.bool,
  embedded: _propTypes.default.bool,
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  value: _propTypes.default.string,
  readOnly: _propTypes.default.bool,
  height: _propTypes.default.oneOfType([_propTypes.default.number, _propTypes.default.string]),
  api: _propTypes.default.func
};
const defaultProps = {
  asText: undefined,
  embedded: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  height: undefined,
  api: undefined
};
/**
 * A text input that accepts and is able to display multiple lines of text. This component takes up
 * more screen space when rendered than LineInput.
 */

class MultilineInput extends _react.Component {
  constructor() {
    super();
    this.handleRef = this.handleRef.bind(this);
  }

  componentDidMount() {
    const {
      api
    } = this.props;

    if (api) {
      api({
        focus: this.focus.bind(this)
      });
    }
  }

  componentWillUnmount() {
    const {
      api
    } = this.props;

    if (api) {
      api(null);
    }
  }

  focus() {
    this.domNode.focus();
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    const {
      api,
      asText,
      embedded,
      height,
      name,
      parentPath,
      subpath,
      value,
      readOnly,
      ...remainingProps
    } = this.props;
    const className = embedded ? _MultilineInput.default.embedded : _MultilineInput.default.normal;
    const normalizedValue = value === null || typeof value === 'undefined' ? '' : value;
    const style = typeof height !== 'undefined' ? {
      height
    } : undefined;

    if (asText) {
      return /*#__PURE__*/_react.default.createElement("div", {
        className: className,
        style: style
      }, normalizedValue);
    }

    return /*#__PURE__*/_react.default.createElement("textarea", _extends({}, remainingProps, {
      className: className,
      "data-name": name,
      disabled: readOnly,
      readOnly: !remainingProps.onChange,
      ref: this.handleRef,
      style: style,
      value: normalizedValue
    }));
  }

}

exports.default = MultilineInput;
MultilineInput.propTypes = propTypes;
MultilineInput.defaultProps = defaultProps;