"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _get = _interopRequireDefault(require("lodash/get"));

var _Menu = _interopRequireDefault(require("./Menu"));

var _parseResourceID = _interopRequireDefault(require("../helpers/parseResourceID"));

var _QuickAdd = _interopRequireDefault(require("../../styles/cspace-input/QuickAdd.css"));

var _QuickAddItem = _interopRequireDefault(require("../../styles/cspace-input/QuickAddItem.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const propTypes = {
  add: _propTypes.default.func,
  displayName: _propTypes.default.string,
  partialTerm: _propTypes.default.string,
  formatAddPrompt: _propTypes.default.func,
  formatCloneOptionLabel: _propTypes.default.func,
  formatCreateNewOptionLabel: _propTypes.default.func,
  formatDestinationName: _propTypes.default.func,
  recordTypes: _propTypes.default.objectOf(_propTypes.default.object),
  showCloneOption: _propTypes.default.bool,
  cloneOptionDisabled: _propTypes.default.bool,
  to: _propTypes.default.string,
  onBeforeItemFocusChange: _propTypes.default.func
};
const defaultProps = {
  add: undefined,
  displayName: undefined,
  partialTerm: undefined,
  formatAddPrompt: (displayName, destinationName) => `Add "${displayName}" to ${destinationName}`,
  formatCloneOptionLabel: () => 'Clone current record',
  formatCreateNewOptionLabel: () => 'Create new record',
  formatDestinationName: (recordTypeConfig, vocabulary) => {
    if (vocabulary) {
      return (0, _get.default)(recordTypeConfig, ['vocabularies', vocabulary, 'messages', 'collectionName', 'defaultMessage']);
    }

    return (0, _get.default)(recordTypeConfig, ['messages', 'record', 'collectionName', 'defaultMessage']);
  },
  recordTypes: undefined,
  showCloneOption: undefined,
  cloneOptionDisabled: undefined,
  to: undefined,
  onBeforeItemFocusChange: undefined
};

class QuickAdd extends _react.Component {
  constructor(props) {
    super(props);
    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);
  }

  focusMenu(itemIndex) {
    if (this.menu) {
      this.menu.focus(itemIndex);
    }
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(item) {
    const {
      add,
      displayName,
      partialTerm
    } = this.props;

    if (add) {
      const {
        clone,
        value
      } = item;
      const typeSpec = clone && value.startsWith('clone:') ? value.substring('clone:'.length) : value;
      const [recordType, vocabulary] = typeSpec.split('/');
      add(recordType, vocabulary, displayName, partialTerm, clone);
    }
  }

  render() {
    const {
      displayName,
      formatAddPrompt,
      formatCloneOptionLabel,
      formatCreateNewOptionLabel,
      formatDestinationName,
      recordTypes,
      showCloneOption,
      cloneOptionDisabled,
      to: destinationID,
      onBeforeItemFocusChange
    } = this.props;
    const destinations = (0, _parseResourceID.default)(destinationID);
    const options = destinations.map(destination => {
      const {
        recordType,
        vocabulary
      } = destination;
      const recordTypeConfig = (0, _get.default)(recordTypes, recordType);

      if (!recordTypeConfig) {
        return null;
      }

      if (vocabulary) {
        const vocabularyConfig = (0, _get.default)(recordTypeConfig, ['vocabularies', vocabulary]);

        if (!vocabularyConfig) {
          return null;
        }
      }

      return {
        label: formatDestinationName(recordTypeConfig, vocabulary),
        value: `${recordType}/${vocabulary}`,
        className: _QuickAddItem.default.add
      };
    }).filter(option => !!option);

    if (options.length === 0) {
      return null;
    } // If showCloneOption is true and there is only one destination, the options should be to
    // create new or clone into the source.


    let singleDestinationName;

    if (options.length === 1 && showCloneOption) {
      singleDestinationName = options[0].label;
      options[0].label = formatCreateNewOptionLabel();
      options.push({ ...options[0],
        clone: true,
        label: formatCloneOptionLabel(),
        className: _QuickAddItem.default.clone,
        disabled: cloneOptionDisabled,
        value: `clone:${options[0].value}`
      });
    }

    return /*#__PURE__*/_react.default.createElement("div", {
      className: _QuickAdd.default.common
    }, /*#__PURE__*/_react.default.createElement("div", null, formatAddPrompt(displayName, singleDestinationName)), /*#__PURE__*/_react.default.createElement(_Menu.default, {
      options: options,
      tabIndex: "-1",
      ref: this.handleMenuRef,
      onBeforeItemFocusChange: onBeforeItemFocusChange,
      onSelect: this.handleMenuSelect
    }));
  }

}

exports.default = QuickAdd;
QuickAdd.propTypes = propTypes;
QuickAdd.defaultProps = defaultProps;