"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _Button = _interopRequireDefault(require("./Button"));

var _LineInput = _interopRequireDefault(require("./LineInput"));

var _RecordTypeInput = _interopRequireDefault(require("./RecordTypeInput"));

var _VocabularyInput = _interopRequireDefault(require("./VocabularyInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _committable = _interopRequireDefault(require("../enhancers/committable"));

var _QuickSearchInput = _interopRequireDefault(require("../../styles/cspace-input/QuickSearchInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const propTypes = {
  formatRecordTypeLabel: _propTypes.default.func,
  formatVocabularyLabel: _propTypes.default.func,
  keywordValue: _propTypes.default.string,
  placeholder: _propTypes.default.string,
  searchButtonLabel: _propTypes.default.string,
  recordTypes: _propTypes.default.objectOf(_propTypes.default.object),
  recordTypeValue: _propTypes.default.string,
  vocabularyValue: _propTypes.default.string,
  onKeywordCommit: _propTypes.default.func,
  onRecordTypeCommit: _propTypes.default.func,
  onVocabularyCommit: _propTypes.default.func,
  search: _propTypes.default.func
};
const defaultProps = {
  formatRecordTypeLabel: undefined,
  formatVocabularyLabel: undefined,
  keywordValue: undefined,
  placeholder: undefined,
  searchButtonLabel: undefined,
  recordTypes: undefined,
  recordTypeValue: undefined,
  vocabularyValue: undefined,
  onKeywordCommit: undefined,
  onRecordTypeCommit: undefined,
  onVocabularyCommit: undefined,
  search: undefined
};
const LineInput = (0, _committable.default)((0, _changeable.default)(_LineInput.default));

class QuickSearchInput extends _react.Component {
  constructor() {
    super();
    this.handleButtonClick = this.handleButtonClick.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleKeywordInputCommit = this.handleKeywordInputCommit.bind(this);
    this.handleRecordTypeDropdownCommit = this.handleRecordTypeDropdownCommit.bind(this);
    this.handleVocabularyDropdownCommit = this.handleVocabularyDropdownCommit.bind(this);
    this.handleVocabularyDropdownUpdate = this.handleVocabularyDropdownUpdate.bind(this);
  }

  handleButtonClick() {
    const {
      search
    } = this.props;

    if (search) {
      search();
    }
  }

  handleKeywordInputCommit(path, value) {
    const {
      onKeywordCommit
    } = this.props;

    if (onKeywordCommit) {
      onKeywordCommit(value);
    }
  }

  handleKeyPress(event) {
    if (event.key === 'Enter') {
      const {
        search
      } = this.props;

      if (search) {
        search();
      }
    }
  }

  handleRecordTypeDropdownCommit(path, value) {
    const {
      onRecordTypeCommit
    } = this.props;

    if (onRecordTypeCommit) {
      onRecordTypeCommit(value);
    }
  }

  handleVocabularyDropdownCommit(path, value) {
    const {
      onVocabularyCommit
    } = this.props;

    if (onVocabularyCommit) {
      onVocabularyCommit(value);
    }
  }

  handleVocabularyDropdownUpdate({
    value
  }) {
    const {
      vocabularyValue,
      onVocabularyCommit
    } = this.props;

    if (onVocabularyCommit && value !== vocabularyValue) {
      onVocabularyCommit(value);
    }
  }

  renderRecordTypeDropdown() {
    const {
      formatRecordTypeLabel,
      recordTypes,
      recordTypeValue
    } = this.props;
    return /*#__PURE__*/_react.default.createElement(_RecordTypeInput.default, {
      embedded: true,
      formatRecordTypeLabel: formatRecordTypeLabel,
      recordTypes: recordTypes,
      value: recordTypeValue,
      onCommit: this.handleRecordTypeDropdownCommit,
      onKeyPress: this.handleKeyPress
    });
  }

  renderVocabularyDropdown() {
    const {
      formatVocabularyLabel,
      recordTypes,
      recordTypeValue,
      vocabularyValue
    } = this.props;
    return /*#__PURE__*/_react.default.createElement(_VocabularyInput.default, {
      embedded: true,
      formatVocabularyLabel: formatVocabularyLabel,
      recordTypes: recordTypes,
      recordType: recordTypeValue,
      value: vocabularyValue,
      onCommit: this.handleVocabularyDropdownCommit,
      onKeyPress: this.handleKeyPress,
      onMount: this.handleVocabularyDropdownUpdate,
      onUpdate: this.handleVocabularyDropdownUpdate
    });
  }

  renderKeywordInput() {
    const {
      keywordValue,
      placeholder
    } = this.props;
    return /*#__PURE__*/_react.default.createElement(LineInput, {
      embedded: true,
      placeholder: placeholder,
      value: keywordValue,
      onCommit: this.handleKeywordInputCommit,
      onKeyPress: this.handleKeyPress
    });
  }

  render() {
    const {
      searchButtonLabel
    } = this.props;
    return /*#__PURE__*/_react.default.createElement("div", {
      className: _QuickSearchInput.default.normal
    }, /*#__PURE__*/_react.default.createElement("div", null, this.renderRecordTypeDropdown(), this.renderVocabularyDropdown(), this.renderKeywordInput()), /*#__PURE__*/_react.default.createElement(_Button.default, {
      "aria-label": searchButtonLabel,
      onClick: this.handleButtonClick
    }));
  }

}

exports.default = QuickSearchInput;
QuickSearchInput.propTypes = propTypes;
QuickSearchInput.defaultProps = defaultProps;