"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _SubstringFilteringDropdownMenuInput = _interopRequireDefault(require("./SubstringFilteringDropdownMenuInput"));

var _DropdownMenuInput = _interopRequireDefault(require("./DropdownMenuInput"));

var _withLabeledOptions = _interopRequireDefault(require("../enhancers/withLabeledOptions"));

var _pathHelpers = require("../helpers/pathHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const propTypes = {
  formatRecordTypeLabel: _propTypes.default.func,
  indentItems: _propTypes.default.bool,
  recordTypes: _propTypes.default.objectOf(_propTypes.default.object),
  rootType: _propTypes.default.string,
  serviceTypes: _propTypes.default.arrayOf(_propTypes.default.string),
  value: _propTypes.default.string,
  filtering: _propTypes.default.bool
};
const defaultProps = {
  filtering: true,
  formatRecordTypeLabel: (name, config) => {
    const {
      messages
    } = config;

    if (messages) {
      return messages.record.collectionName.defaultMessage;
    }

    return name;
  },
  indentItems: true,
  recordTypes: {},
  rootType: 'all',
  serviceTypes: ['object', 'procedure', 'authority'],
  value: undefined
};
const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions.default)(_SubstringFilteringDropdownMenuInput.default);
const DropdownMenuInput = (0, _withLabeledOptions.default)(_DropdownMenuInput.default);

class RecordTypeInput extends _react.Component {
  constructor(props) {
    super(props);
    this.state = {};
  }

  componentDidMount() {
    this.initOptions(this.props);
  }

  componentWillReceiveProps(nextProps) {
    const {
      formatRecordTypeLabel,
      indentItems,
      recordTypes,
      rootType,
      serviceTypes
    } = this.props;
    const {
      formatRecordTypeLabel: nextFormatRecordTypeLabel,
      indentItems: nextIndentItems,
      recordTypes: nextRecordTypes,
      rootType: nextRootType,
      serviceTypes: nextServiceTypes
    } = nextProps;

    if (formatRecordTypeLabel !== nextFormatRecordTypeLabel || indentItems !== nextIndentItems || recordTypes !== nextRecordTypes || rootType !== nextRootType || serviceTypes !== nextServiceTypes) {
      this.initOptions(nextProps);
    }
  }

  initOptions(props) {
    const {
      formatRecordTypeLabel,
      indentItems,
      recordTypes,
      rootType,
      serviceTypes,
      value
    } = props;
    const options = [];

    if (recordTypes[rootType]) {
      options.push({
        value: rootType,
        label: formatRecordTypeLabel(rootType, recordTypes[rootType])
      });
    }

    serviceTypes.forEach(serviceType => {
      const recordTypeNames = Object.keys(recordTypes).filter(recordTypeName => {
        const recordTypeConfig = recordTypes[recordTypeName];
        return recordTypeConfig && recordTypeConfig.serviceConfig.serviceType === serviceType && !recordTypeConfig.disabled;
      }).sort((nameA, nameB) => {
        const configA = recordTypes[nameA];
        const configB = recordTypes[nameB]; // Primary sort by sortOrder

        let sortOrderA = configA.sortOrder;
        let sortOrderB = configB.sortOrder;

        if (typeof sortOrderA !== 'number') {
          sortOrderA = Number.MAX_VALUE;
        }

        if (typeof sortOrderB !== 'number') {
          sortOrderB = Number.MAX_VALUE;
        }

        if (sortOrderA !== sortOrderB) {
          return sortOrderA > sortOrderB ? 1 : -1;
        } // Secondary sort by label


        const labelA = formatRecordTypeLabel(nameA, configA);
        const labelB = formatRecordTypeLabel(nameB, configB); // FIXME: This should be locale aware

        return labelA.localeCompare(labelB);
      });
      let indent = 0;

      if (recordTypeNames.length > 1 && recordTypes[serviceType]) {
        // If there is more than one record type within this service type, and the service type is
        // itself a record type, add the service type as a parent option.
        if (indentItems) {
          indent += 1;
        }

        options.push({
          indent,
          value: serviceType,
          label: formatRecordTypeLabel(serviceType, recordTypes[serviceType])
        });
      }

      if (indentItems) {
        indent += 1;
      }

      recordTypeNames.forEach(recordTypeName => {
        options.push({
          indent,
          value: recordTypeName,
          label: formatRecordTypeLabel(recordTypeName, recordTypes[recordTypeName])
        });
      });
    });
    this.setState({
      options
    });

    if (typeof value === 'undefined') {
      let defaultValue;

      for (let i = 0; i < options.length; i += 1) {
        const recordTypeName = options[i].value;
        const recordType = recordTypes[recordTypeName];

        if (recordType.defaultForSearch) {
          defaultValue = recordTypeName;
          break;
        }
      }

      if (!defaultValue && options.length > 0) {
        defaultValue = options[0].value;
      }

      if (defaultValue) {
        const {
          onCommit
        } = props;

        if (onCommit) {
          onCommit((0, _pathHelpers.getPath)(props), defaultValue);
        }
      }
    }
  }

  render() {
    const {
      filtering,
      formatRecordTypeLabel,
      indentItems,
      recordTypes,
      rootType,
      serviceTypes,
      ...remainingProps
    } = this.props;
    const {
      options
    } = this.state;
    const Input = filtering ? SubstringFilteringDropdownMenuInput : DropdownMenuInput;
    return /*#__PURE__*/_react.default.createElement(Input, _extends({
      blankable: false,
      options: options // eslint-disable-next-line react/jsx-props-no-spreading

    }, remainingProps));
  }

}

exports.default = RecordTypeInput;
RecordTypeInput.propTypes = propTypes;
RecordTypeInput.defaultProps = defaultProps;