"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _immutable = _interopRequireDefault(require("immutable"));

var _get = _interopRequireDefault(require("lodash/get"));

var _MiniButton = _interopRequireDefault(require("./MiniButton"));

var _normalizeLabel = _interopRequireDefault(require("../helpers/normalizeLabel"));

var _pathHelpers = require("../helpers/pathHelpers");

var _MiniButtonContainer = _interopRequireDefault(require("../../styles/cspace-input/MiniButtonContainer.css"));

var _MoveToTopButton = _interopRequireDefault(require("../../styles/cspace-input/MoveToTopButton.css"));

var _RepeatingInput = _interopRequireDefault(require("../../styles/cspace-input/RepeatingInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const propTypes = {
  children: _propTypes.default.node,
  name: _propTypes.default.string,

  /* eslint-disable react/no-unused-prop-types */
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  value: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.object, _propTypes.default.instanceOf(_immutable.default.List), _propTypes.default.arrayOf(_propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.object]))]),
  readOnly: _propTypes.default.bool,
  asText: _propTypes.default.bool,
  reorderable: _propTypes.default.bool,
  disableRemoveButton: _propTypes.default.func,
  renderOrderIndicator: _propTypes.default.func,
  onAddInstance: _propTypes.default.func,
  onCommit: _propTypes.default.func,
  onMoveInstance: _propTypes.default.func,
  onRemoveInstance: _propTypes.default.func
};
const defaultProps = {
  children: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  asText: undefined,
  reorderable: true,
  disableRemoveButton: undefined,
  renderOrderIndicator: undefined,
  onAddInstance: undefined,
  onCommit: undefined,
  onMoveInstance: undefined,
  onRemoveInstance: undefined
};

const normalizeValue = value => {
  const defaultValue = [undefined];

  if (!value) {
    return defaultValue;
  }

  let normalized;

  if (Array.isArray(value)) {
    normalized = value;
  } else if (_immutable.default.List.isList(value)) {
    normalized = value.toArray();
  } else {
    normalized = [value];
  }

  if (normalized.length === 0) {
    return defaultValue;
  }

  return normalized;
};

const renderHeader = label => label;

class RepeatingInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleInstanceCommit = this.handleInstanceCommit.bind(this);
    this.handleAddButtonClick = this.handleAddButtonClick.bind(this);
    this.handleMoveToTopButtonClick = this.handleMoveToTopButtonClick.bind(this);
    this.handleMoveToTopButtonKeyDown = this.handleMoveToTopButtonKeyDown.bind(this);
    this.handleInstancesContainerRef = this.handleInstancesContainerRef.bind(this);
    this.handleRemoveButtonClick = this.handleRemoveButtonClick.bind(this);
  }

  getLabel() {
    const {
      children
    } = this.props;

    const template = _react.default.Children.only(children);

    const {
      label
    } = template.props;
    return (0, _normalizeLabel.default)(label);
  }

  focus(position) {
    if (this.instancesContainerNode) {
      // FIXME: This is a bit of a hack. It should use refs to track the move to top button nodes.
      const instance = this.instancesContainerNode.querySelector(`div[data-instancename="${position}"]`);
      const moveToTopButtonInstance = instance.querySelector('.cspace-input-MoveToTopButton--common');

      if (moveToTopButtonInstance) {
        moveToTopButtonInstance.focus();
      }
    }
  }

  handleAddButtonClick() {
    const {
      onAddInstance
    } = this.props;

    if (onAddInstance) {
      onAddInstance((0, _pathHelpers.getPath)(this.props));
    }
  }

  handleInstanceCommit(instancePath, value) {
    const {
      onCommit
    } = this.props;

    if (onCommit) {
      onCommit(instancePath, value);
    }
  }

  handleInstancesContainerRef(ref) {
    this.instancesContainerNode = ref;
  }

  handleMoveToTopButtonClick(event) {
    const {
      onMoveInstance
    } = this.props;

    if (onMoveInstance) {
      const instanceName = event.target.dataset.instancename;
      const newPosition = 0;
      onMoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName], newPosition);
    }
  }

  handleMoveToTopButtonKeyDown(event) {
    const {
      value,
      onAddInstance,
      onMoveInstance
    } = this.props;

    if (event.shiftKey) {
      const {
        key,
        currentTarget
      } = event;

      if (key === 'ArrowDown' || key === 'ArrowUp') {
        if (onMoveInstance) {
          const instanceName = currentTarget.dataset.instancename;
          const position = parseInt(instanceName, 10);
          const normalizedValue = normalizeValue(value);
          let newPosition;

          if (key === 'ArrowDown') {
            newPosition = position + 1;

            if (newPosition > normalizedValue.length - 1) {
              newPosition = 0;
            }
          } else {
            newPosition = position - 1;

            if (newPosition < 0) {
              newPosition = normalizedValue.length - 1;
            }
          }

          onMoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName], newPosition);
          this.focus(newPosition);
        }
      } else if (key === '+') {
        if (onAddInstance) {
          const instanceName = currentTarget.dataset.instancename;
          const position = parseInt(instanceName, 10);
          onAddInstance((0, _pathHelpers.getPath)(this.props), position + 1);
        }
      }
    }
  }

  handleRemoveButtonClick(event) {
    const {
      onRemoveInstance
    } = this.props;

    if (onRemoveInstance) {
      const instanceName = event.target.dataset.instancename;
      onRemoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName]);
    }
  }

  renderEmbeddedHeader(label) {
    const {
      readOnly
    } = this.props;
    let removeButtonCol;

    if (!readOnly) {
      removeButtonCol = /*#__PURE__*/_react.default.createElement("div", null);
    }

    return /*#__PURE__*/_react.default.createElement("header", null, /*#__PURE__*/_react.default.createElement("div", null), /*#__PURE__*/_react.default.createElement("div", null, label), removeButtonCol);
  }

  renderInstances() {
    const {
      asText,
      children,
      value,
      readOnly,
      reorderable,
      disableRemoveButton,
      renderOrderIndicator
    } = this.props;

    const template = _react.default.Children.only(children);

    const normalizedValue = normalizeValue(value);
    return normalizedValue.map((instanceValue, index, list) => {
      const instanceName = `${index}`;
      const overrideProps = {
        asText,
        readOnly,
        embedded: true,
        label: undefined,
        name: instanceName,
        parentPath: (0, _pathHelpers.getPath)(this.props),
        value: instanceValue,
        // The template is expected to accept an onCommit prop.
        onCommit: this.handleInstanceCommit
      };

      const inputInstance = /*#__PURE__*/_react.default.cloneElement(template, overrideProps);

      let orderIndicator = null;
      const hideOrderNumber = readOnly && normalizedValue.length === 1 && !normalizedValue[0];
      const orderNumber = hideOrderNumber ? null : index + 1;

      if (renderOrderIndicator) {
        orderIndicator = renderOrderIndicator(orderNumber);
      } else {
        const className = readOnly ? _MiniButtonContainer.default.readOnly : _MiniButtonContainer.default.normal;

        if (asText) {
          orderIndicator = /*#__PURE__*/_react.default.createElement("div", null, orderNumber);
        } else {
          orderIndicator = /*#__PURE__*/_react.default.createElement("div", {
            className: className
          }, /*#__PURE__*/_react.default.createElement(_MiniButton.default, {
            className: _MoveToTopButton.default.common,
            "data-name": "moveToTop",
            "data-instancename": instanceName,
            disabled: !reorderable,
            readOnly: readOnly,
            onClick: this.handleMoveToTopButtonClick,
            onKeyDown: this.handleMoveToTopButtonKeyDown
          }, orderNumber));
        }
      }

      let removeButton;

      if (!readOnly && !asText) {
        const className = _MiniButtonContainer.default.normal;
        removeButton = /*#__PURE__*/_react.default.createElement("div", {
          className: className
        }, /*#__PURE__*/_react.default.createElement(_MiniButton.default, {
          "data-instancename": instanceName,
          "data-name": "remove",
          disabled: list.length < 2 || disableRemoveButton && disableRemoveButton(instanceValue),
          onClick: this.handleRemoveButtonClick
        }, "\u2212"));
      }

      return /*#__PURE__*/_react.default.createElement("div", {
        "data-instancename": instanceName // It's unclear what can be used as a unique key here. It might be safe to use value? But
        // value isn't necessarily a string, so it would have to be JSON.stringify(value), which
        // could be really inefficient.
        // eslint-disable-next-line react/no-array-index-key
        ,
        key: index
      }, orderIndicator, /*#__PURE__*/_react.default.createElement("div", null, inputInstance), removeButton);
    });
  }

  render() {
    const {
      asText,
      name,
      readOnly
    } = this.props;
    const label = this.getLabel();
    const isLabelEmbedded = (0, _get.default)(label, ['props', 'embedded']);
    const instances = this.renderInstances();

    if (asText) {
      return /*#__PURE__*/_react.default.createElement("div", null, instances);
    }

    let footer;

    if (!readOnly) {
      const className = _MiniButtonContainer.default.normal;
      footer = /*#__PURE__*/_react.default.createElement("footer", null, /*#__PURE__*/_react.default.createElement("div", {
        className: className
      }, /*#__PURE__*/_react.default.createElement(_MiniButton.default, {
        "data-name": "add",
        onClick: this.handleAddButtonClick
      }, "+")));
    }

    const className = readOnly ? _RepeatingInput.default.readOnly : _RepeatingInput.default.normal;
    return /*#__PURE__*/_react.default.createElement("fieldset", {
      className: className,
      "data-name": name
    }, isLabelEmbedded ? null : renderHeader(label), /*#__PURE__*/_react.default.createElement("div", {
      ref: this.handleInstancesContainerRef
    }, isLabelEmbedded ? this.renderEmbeddedHeader(label) : null, instances), footer);
  }

}

exports.default = RepeatingInput;
RepeatingInput.propTypes = propTypes;
RepeatingInput.defaultProps = defaultProps;