"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _reactQuill = _interopRequireWildcard(require("react-quill"));

require("!style-loader!css-loader!react-quill/dist/quill.core.css");

var _classnames = _interopRequireDefault(require("classnames"));

var _pathHelpers = require("../helpers/pathHelpers");

var _RichTextInput = _interopRequireDefault(require("../../styles/cspace-input/RichTextInput.css"));

require("!style-loader!css-loader!../../styles/react-quill/cspace.css");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* global window */
// eslint-disable-next-line import/no-webpack-loader-syntax, import/no-unresolved
// eslint-disable-next-line import/no-webpack-loader-syntax, import/no-unresolved
const propTypes = {
  embedded: _propTypes.default.bool,
  multiline: _propTypes.default.bool,

  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  value: _propTypes.default.string,
  readOnly: _propTypes.default.bool,
  onCommit: _propTypes.default.func
};
const defaultProps = {
  embedded: undefined,
  multiline: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  onCommit: undefined
}; // Change bold rendering to <b> instead of <strong>, since it is more semantically neutral.

const Bold = _reactQuill.Quill.import('formats/bold');

Bold.tagName = 'B';

_reactQuill.Quill.register(Bold, true); // Change italic rendering to <i> instead of <em>, since it is more semantically neutral.


const Italic = _reactQuill.Quill.import('formats/italic');

Italic.tagName = 'I';

_reactQuill.Quill.register(Italic, true);

const normalizeValue = value => {
  let normalizedValue = value;

  if (normalizedValue) {
    // If there is only a single paragraph, remove the <p></p> wrapper.
    if (normalizedValue.indexOf('<p>', 1) < 0) {
      normalizedValue = normalizedValue.replace(/^<p>|<\/p>$/g, '');
    } // If content is '<br>', set it to empty. Quill uses this as a placeholder.


    if (normalizedValue === '<br>') {
      normalizedValue = '';
    }
  }

  return normalizedValue;
};

const preventButtonMouseDown = event => {
  if (event.target.nodeName === 'BUTTON') {
    // On mouseDown on a toolbar button, Firefox fires a blur event with null relatedTarget. This
    // would be fine, if relatedTarget were correctly set to the toolbar button. But since it's
    // null, the toolbar disappears. Prevent this.
    event.preventDefault();
  }
};

class RichTextInput extends _react.Component {
  constructor(props) {
    super(props);
    const {
      multiline,
      value
    } = this.props; // Set the allowed formats.
    // TODO: Make this a prop.
    // TODO: Add paragraph formatting (e.g. lists) when multiline is true?

    this.formats = ['bold', 'italic', 'underline', 'script']; // Configure the toolbar buttons.
    // TODO: Make this a prop.

    this.modules = {
      keyboard: {
        bindings: {
          // Only allow newlines in content if multiline prop is true.
          enter: {
            key: 13,
            handler: () => {
              this.commit();
              return !!multiline;
            }
          },
          // Don't allow tabs in the content, just do the browser default of focusing the next
          // field.
          tab: null
        }
      },
      toolbar: ['bold', 'italic', 'underline', {
        script: 'sub'
      }, {
        script: 'super'
      }, 'clean']
    }; // TODO: Make a custom HTML toolbar for accessibility:
    // https://github.com/zenoamaro/react-quill#html-toolbar

    this.handleBlur = this.handleBlur.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleRef = this.handleRef.bind(this);
    this.state = {
      value
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  shouldComponentUpdate(nextProps, nextState) {
    const {
      embedded,
      multiline,
      readOnly,
      value
    } = this.props;
    const {
      focused
    } = this.state;
    return nextProps.embedded !== embedded || nextProps.multiline !== multiline || nextProps.readOnly !== readOnly || nextProps.value !== value || nextState.focused !== focused;
  }

  commit() {
    const {
      onCommit,
      value: prevValue
    } = this.props;

    if (onCommit) {
      const {
        value
      } = this.state;
      const nextValue = normalizeValue(value);

      if (nextValue !== prevValue) {
        onCommit((0, _pathHelpers.getPath)(this.props), nextValue);
      }
    }
  }

  handleBlur(event) {
    if (!this.domNode.contains(event.relatedTarget)) {
      window.setTimeout(() => {
        this.setState({
          focused: false
        });
        this.commit();
      }, 0);
    }
  }

  handleChange(value) {
    this.setState({
      value
    });
  }

  handleFocus(event) {
    if (this.domNode && !this.domNode.contains(event.relatedTarget)) {
      this.setState({
        focused: true
      });
    }
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    const {
      embedded,
      multiline,
      readOnly
    } = this.props;
    const {
      focused,
      value
    } = this.state;
    const normalizedValue = value || '';
    const className = (0, _classnames.default)({
      [_RichTextInput.default.embedded]: embedded,
      [_RichTextInput.default.normal]: !embedded,
      [_RichTextInput.default.enabled]: !readOnly,
      [_RichTextInput.default.multiline]: multiline,
      [_RichTextInput.default.focus]: focused
    }); // The react-quill onBlur event is unreliable, so the wrapper div here is used to do our own
    // implementation.
    // https://github.com/zenoamaro/react-quill/issues/286
    // https://github.com/zenoamaro/react-quill/issues/276

    return (
      /*#__PURE__*/
      // eslint-disable-next-line jsx-a11y/no-static-element-interactions
      _react.default.createElement("div", {
        ref: this.handleRef,
        onBlur: this.handleBlur,
        onFocus: this.handleFocus,
        onMouseDown: preventButtonMouseDown
      }, /*#__PURE__*/_react.default.createElement(_reactQuill.default, {
        className: className,
        formats: this.formats,
        modules: this.modules,
        readOnly: readOnly,
        theme: "snow",
        value: normalizedValue,
        onChange: this.handleChange
      }))
    );
  }

}

exports.default = RichTextInput;
RichTextInput.propTypes = propTypes;
RichTextInput.defaultProps = defaultProps;