"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _immutable = _interopRequireDefault(require("immutable"));

var _Row = _interopRequireDefault(require("cspace-layout/lib/components/Row"));

var _set = _interopRequireDefault(require("lodash/set"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _CustomCompoundInput = _interopRequireDefault(require("./CustomCompoundInput"));

var _DropdownInput = _interopRequireDefault(require("./DropdownInput"));

var _Label = _interopRequireDefault(require("./Label"));

var _TextInput = _interopRequireDefault(require("./TextInput"));

var _SubstringFilteringDropdownMenuInput = _interopRequireDefault(require("./SubstringFilteringDropdownMenuInput"));

var _TermPickerInput = _interopRequireDefault(require("./TermPickerInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _committable = _interopRequireDefault(require("../enhancers/committable"));

var _withLabeledOptions = _interopRequireDefault(require("../enhancers/withLabeledOptions"));

var _labelable = _interopRequireDefault(require("../enhancers/labelable"));

var _pathHelpers = require("../helpers/pathHelpers");

var _dateHelpers = require("../helpers/dateHelpers");

var _StructuredDateInput = _interopRequireDefault(require("../../styles/cspace-input/StructuredDateInput.css"));

var _Message = _interopRequireDefault(require("../../styles/cspace-input/Message.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const fieldLabels = {
  earliestSingle: 'Earliest/Single',
  latest: 'Latest',
  datePeriod: 'Period',
  dateAssociation: 'Association',
  dateNote: 'Note',
  dateYear: 'Year',
  dateMonth: 'Month',
  dateDay: 'Day',
  dateEra: 'Era',
  dateCertainty: 'Certainty',
  dateQualifier: 'Qualifier',
  dateQualifierValue: 'Value',
  dateQualifierUnit: 'Unit'
};
const propTypes = {
  defaultValue: _propTypes.default.oneOfType([_propTypes.default.object, _propTypes.default.instanceOf(_immutable.default.Map)]),
  embedded: _propTypes.default.bool,
  name: _propTypes.default.string,
  optionLists: _propTypes.default.objectOf(_propTypes.default.array),
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  terms: _propTypes.default.objectOf(_propTypes.default.array),
  value: _propTypes.default.oneOfType([_propTypes.default.object, _propTypes.default.instanceOf(_immutable.default.Map)]),
  readOnly: _propTypes.default.bool,
  formatFieldLabel: _propTypes.default.func,
  formatOptionLabel: _propTypes.default.func,
  formatParseFailedMessage: _propTypes.default.func,
  parseDisplayDate: _propTypes.default.func,
  onCommit: _propTypes.default.func,
  onMount: _propTypes.default.func
};
const defaultProps = {
  defaultValue: {},
  embedded: undefined,
  formatFieldLabel: name => fieldLabels[name],
  formatOptionLabel: undefined,
  formatParseFailedMessage: () => 'Unrecognized display date format. Try a different format, or enter values in the fields below.',
  name: undefined,
  onCommit: undefined,
  onMount: undefined,
  parentPath: undefined,
  parseDisplayDate: undefined,
  optionLists: {},
  readOnly: undefined,
  subpath: undefined,
  terms: {},
  value: undefined
};
const DropdownInput = (0, _committable.default)((0, _changeable.default)(_DropdownInput.default));
const TextInput = (0, _committable.default)((0, _changeable.default)(_TextInput.default));
const LabelableTextInput = (0, _labelable.default)(TextInput);
const LabelableTermPickerInput = (0, _labelable.default)(_TermPickerInput.default);
const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions.default)(_SubstringFilteringDropdownMenuInput.default);
const LabelableSubstringFilteringDropdownMenuInput = (0, _labelable.default)(SubstringFilteringDropdownMenuInput);
const primaryFieldName = 'dateDisplayDate';

const getStructDateFieldValue = (structDateValue, fieldName) => {
  if (!structDateValue) {
    return undefined;
  }

  if (_immutable.default.Map.isMap(structDateValue)) {
    return structDateValue.get(fieldName);
  }

  return structDateValue[fieldName];
};

const getPrimaryValue = structDateValue => getStructDateFieldValue(structDateValue, primaryFieldName);

class StructuredDateInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleInputCommit = this.handleInputCommit.bind(this);
    this.handlePrimaryInputChange = this.handlePrimaryInputChange.bind(this);
    this.handlePrimaryInputCommit = this.handlePrimaryInputCommit.bind(this);
    const value = props.value || props.defaultValue;
    this.state = {
      value,
      open: false,
      primaryValue: getPrimaryValue(value)
    };
  }

  componentDidMount() {
    const {
      onMount
    } = this.props;

    if (onMount) {
      onMount();
    }
  }

  componentWillReceiveProps(nextProps) {
    const value = nextProps.value || nextProps.defaultValue;
    this.setState({
      value,
      primaryValue: getPrimaryValue(value)
    });
  }

  getValue(name) {
    const {
      value
    } = this.state;
    return getStructDateFieldValue(value, name);
  }

  setValue(path, value) {
    const {
      value: structDateValue
    } = this.state;
    let newStructDateValue;

    if (_immutable.default.Map.isMap(structDateValue)) {
      if (path.length === 0) {
        newStructDateValue = structDateValue.merge(value);
      } else {
        newStructDateValue = structDateValue.setIn(path, value);
      }

      const newStructDateValueAsObject = newStructDateValue.toJS();
      const dateEarliestScalarValue = (0, _dateHelpers.computeEarliestScalarDate)(newStructDateValueAsObject);
      const dateLatestScalarValue = (0, _dateHelpers.computeLatestScalarDate)(newStructDateValueAsObject);
      newStructDateValue = newStructDateValue.set('dateEarliestScalarValue', dateEarliestScalarValue);
      newStructDateValue = newStructDateValue.set('dateLatestScalarValue', dateLatestScalarValue);
      newStructDateValue = newStructDateValue.set('scalarValuesComputed', true);
    } else {
      if (path.length === 0) {
        newStructDateValue = (0, _merge.default)(structDateValue, value);
      } else {
        newStructDateValue = (0, _merge.default)({}, structDateValue);
        (0, _set.default)(newStructDateValue, path, value);
      }

      const dateEarliestScalarValue = (0, _dateHelpers.computeEarliestScalarDate)(newStructDateValue);
      const dateLatestScalarValue = (0, _dateHelpers.computeLatestScalarDate)(newStructDateValue);
      newStructDateValue.dateEarliestScalarValue = dateEarliestScalarValue;
      newStructDateValue.dateLatestScalarValue = dateLatestScalarValue;
      newStructDateValue.scalarValuesComputed = true;
    }

    return newStructDateValue;
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });
  }

  handleInputCommit(path, value) {
    const nextStructDateValue = this.setValue(path, value);
    const {
      onCommit,
      parseDisplayDate
    } = this.props;
    const {
      value: prevStructDateValue
    } = this.state;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), nextStructDateValue);
    }

    if (parseDisplayDate) {
      const prevDisplayDate = getStructDateFieldValue(prevStructDateValue, 'dateDisplayDate');
      const nextDisplayDate = getStructDateFieldValue(nextStructDateValue, 'dateDisplayDate');

      if (nextDisplayDate !== prevDisplayDate) {
        this.setState({
          parseFailed: false
        });
        parseDisplayDate(nextDisplayDate).then(result => {
          if (!result || result.isError) {
            this.setState({
              parseFailed: true
            });
          } else {
            this.handleInputCommit([], result.value);
          }
        });
      }
    }

    this.setState({
      value: nextStructDateValue
    });
  }

  handlePrimaryInputCommit(path, value) {
    const initialValue = this.getValue(primaryFieldName);

    if ((value || initialValue) && value !== initialValue) {
      this.handleInputCommit([primaryFieldName], value);
    }
  }

  handlePrimaryInputChange(value) {
    this.setState({
      open: true,
      primaryValue: value
    });
  }

  renderParseFailedMessage() {
    const {
      parseFailed
    } = this.state;

    if (!parseFailed) {
      return null;
    }

    const {
      formatParseFailedMessage
    } = this.props;
    return /*#__PURE__*/_react.default.createElement("div", {
      className: _Message.default.warning
    }, formatParseFailedMessage());
  }

  renderDropdownInput(fieldName, listName, embedded = false) {
    // Render an appropriate dropdown input for the given list name. If the name is the name of a
    // term list provided in the terms prop, a TermPickerInput is rendered. Otherwise, if the name
    // is the name of an option list provided in the optionLists prop, a
    // SubstringFilteringDropdownMenuInput is rendered.
    const {
      formatFieldLabel,
      formatOptionLabel,
      optionLists,
      terms
    } = this.props;
    const matchingTermList = terms[listName];

    if (matchingTermList) {
      if (embedded) {
        return /*#__PURE__*/_react.default.createElement(_TermPickerInput.default, {
          embedded: true,
          name: fieldName,
          terms: matchingTermList,
          onCommit: this.handleInputCommit
        });
      }

      return /*#__PURE__*/_react.default.createElement(LabelableTermPickerInput, {
        label: formatFieldLabel(fieldName),
        name: fieldName,
        terms: matchingTermList,
        onCommit: this.handleInputCommit
      });
    }

    const matchingOptionList = optionLists[listName];

    if (matchingOptionList) {
      if (embedded) {
        return /*#__PURE__*/_react.default.createElement(SubstringFilteringDropdownMenuInput, {
          embedded: true,
          formatOptionLabel: formatOptionLabel,
          name: fieldName,
          options: matchingOptionList,
          onCommit: this.handleInputCommit
        });
      }

      return /*#__PURE__*/_react.default.createElement(LabelableSubstringFilteringDropdownMenuInput, {
        formatOptionLabel: formatOptionLabel,
        label: formatFieldLabel(fieldName),
        name: fieldName,
        options: matchingOptionList,
        onCommit: this.handleInputCommit
      });
    }

    if (embedded) {
      return /*#__PURE__*/_react.default.createElement(TextInput, {
        embedded: true,
        name: fieldName,
        onCommit: this.handleInputCommit
      });
    }

    return /*#__PURE__*/_react.default.createElement(LabelableTextInput, {
      name: fieldName,
      label: formatFieldLabel(fieldName),
      onCommit: this.handleInputCommit
    });
  }

  render() {
    const {
      defaultValue,
      formatFieldLabel,
      formatOptionLabel,
      formatParseFailedMessage,
      name,
      optionLists,
      parentPath,
      subpath,
      terms,
      value: valueProp,
      onCommit,
      onMount,
      parseDisplayDate,
      ...remainingProps
    } = this.props;
    const {
      readOnly
    } = remainingProps;
    const {
      open,
      primaryValue,
      value
    } = this.state;
    return /*#__PURE__*/_react.default.createElement(DropdownInput // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      className: readOnly ? _StructuredDateInput.default.readOnly : _StructuredDateInput.default.normal,
      commitUnchanged: true,
      open: open,
      value: primaryValue,
      isOpenableWhenReadOnly: true,
      onChange: this.handlePrimaryInputChange,
      onClose: this.handleDropdownInputClose,
      onCommit: this.handlePrimaryInputCommit,
      onOpen: this.handleDropdownInputOpen
    }), /*#__PURE__*/_react.default.createElement(_CustomCompoundInput.default, {
      readOnly: readOnly,
      value: value
    }, /*#__PURE__*/_react.default.createElement(_Row.default, null, /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement(LabelableTextInput, {
      name: "datePeriod",
      label: formatFieldLabel('datePeriod'),
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("div", null, this.renderDropdownInput('dateAssociation', 'dateassociation')), /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement(LabelableTextInput, {
      name: "dateNote",
      label: formatFieldLabel('dateNote'),
      onCommit: this.handleInputCommit
    }))), this.renderParseFailedMessage(), /*#__PURE__*/_react.default.createElement("table", null, /*#__PURE__*/_react.default.createElement("thead", null, /*#__PURE__*/_react.default.createElement("tr", null, /*#__PURE__*/_react.default.createElement("td", null), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateYear'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateMonth'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateDay'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateEra'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateCertainty'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateQualifier'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateQualifierValue'))), /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('dateQualifierUnit'))))), /*#__PURE__*/_react.default.createElement("tbody", null, /*#__PURE__*/_react.default.createElement("tr", null, /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('earliestSingle'))), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateEarliestSingleYear",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateEarliestSingleMonth",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateEarliestSingleDay",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateEarliestSingleEra', 'dateera', true)), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateEarliestSingleCertainty', 'datecertainty', true)), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateEarliestSingleQualifier', 'dateQualifiers', true)), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateEarliestSingleQualifierValue",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateEarliestSingleQualifierUnit', 'datequalifier', true))), /*#__PURE__*/_react.default.createElement("tr", null, /*#__PURE__*/_react.default.createElement("th", null, /*#__PURE__*/_react.default.createElement(_Label.default, null, formatFieldLabel('latest'))), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateLatestYear",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateLatestMonth",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateLatestDay",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateLatestEra', 'dateera', true)), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateLatestCertainty', 'datecertainty', true)), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateLatestQualifier', 'dateQualifiers', true)), /*#__PURE__*/_react.default.createElement("td", null, /*#__PURE__*/_react.default.createElement(TextInput, {
      embedded: true,
      name: "dateLatestQualifierValue",
      onCommit: this.handleInputCommit
    })), /*#__PURE__*/_react.default.createElement("td", null, this.renderDropdownInput('dateLatestQualifierUnit', 'datequalifier', true)))))));
  }

}

exports.default = StructuredDateInput;
StructuredDateInput.propTypes = propTypes;
StructuredDateInput.defaultProps = defaultProps;