"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = TextInput;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _LineInput = _interopRequireDefault(require("./LineInput"));

var _MultilineInput = _interopRequireDefault(require("./MultilineInput"));

var _pathHelpers = require("../helpers/pathHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// FIXME: Rationalize these propTypes. Make consistent across LineInput, MultilineInput, and
// TextInput.
const propTypes = {
  'aria-label': _propTypes.default.string,
  autoComplete: _propTypes.default.string,
  embedded: _propTypes.default.bool,
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  value: _propTypes.default.string,
  readOnly: _propTypes.default.bool,
  multiline: _propTypes.default.bool
};
const defaultProps = {
  'aria-label': undefined,
  autoComplete: undefined,
  embedded: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  value: undefined,
  readOnly: undefined,
  multiline: null
};

function getBaseComponent(multiline, value) {
  let isMultiline = false;

  if (typeof multiline === 'undefined' || multiline == null) {
    isMultiline = value && value.indexOf('\n') >= 0;
  } else if (multiline) {
    isMultiline = true;
  }

  return isMultiline ? _MultilineInput.default : _LineInput.default;
}
/**
 * A text input. There are three modes: single line, multiline, and fallback.
 *
 * In single line mode, the input is rendered as a LineInput.
 *
 * In multiline mode, the input is rendered as a MultilineInput.
 *
 * In fallback mode, the input is rendered as a LineInput, unless the supplied value contains a
 * newline; then it is rendered as a MultilineInput. Fallback mode is appropriate when existing
 * data to be rendered may contain a mix of single line and multiline values, but new values
 * should be prohibited from containing multiple lines.
 */


function TextInput(props) {
  const {
    multiline,
    ...remainingProps
  } = props;
  const {
    value
  } = remainingProps;
  const BaseComponent = getBaseComponent(multiline, value);
  return /*#__PURE__*/_react.default.createElement(BaseComponent // eslint-disable-next-line react/jsx-props-no-spreading
  , remainingProps);
}

TextInput.propTypes = propTypes;
TextInput.defaultProps = defaultProps;