"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _TextInput = _interopRequireDefault(require("./TextInput"));

var _DropdownMenuInput = _interopRequireDefault(require("./DropdownMenuInput"));

var _FileInput = _interopRequireDefault(require("./FileInput"));

var _changeable = _interopRequireDefault(require("../enhancers/changeable"));

var _committable = _interopRequireDefault(require("../enhancers/committable"));

var _labelable = _interopRequireDefault(require("../enhancers/labelable"));

var _pathHelpers = require("../helpers/pathHelpers");

var _UploadInput = _interopRequireDefault(require("../../styles/cspace-input/UploadInput.css"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* global File */
const DropdownMenuInput = (0, _labelable.default)(_DropdownMenuInput.default);
const TextInput = (0, _labelable.default)((0, _committable.default)((0, _changeable.default)(_TextInput.default)));
const FileInput = (0, _labelable.default)(_FileInput.default);
const propTypes = {
  accept: _propTypes.default.string,

  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,

  /* eslint-enable react/no-unused-prop-types */
  type: _propTypes.default.string,
  value: _propTypes.default.oneOfType([_propTypes.default.arrayOf(_propTypes.default.instanceOf(File)), _propTypes.default.string]),
  readOnly: _propTypes.default.bool,
  typeInputLabel: _propTypes.default.string,
  fileOptionLabel: _propTypes.default.string,
  urlOptionLabel: _propTypes.default.string,
  fileInputLabel: _propTypes.default.string,
  fileChooseButtonLabel: _propTypes.default.string,
  urlInputLabel: _propTypes.default.string,
  formatFileInfo: _propTypes.default.func,
  onCommit: _propTypes.default.func,
  onTypeChanged: _propTypes.default.func
};
const defaultProps = {
  accept: undefined,
  name: undefined,
  parentPath: undefined,
  subpath: undefined,
  readOnly: undefined,
  type: 'file',
  typeInputLabel: 'Upload',
  fileOptionLabel: 'local file',
  urlOptionLabel: 'external media',
  fileInputLabel: 'File',
  fileChooseButtonLabel: undefined,
  urlInputLabel: 'URL',
  value: undefined,
  formatFileInfo: undefined,
  onCommit: undefined,
  onTypeChanged: undefined
};

class UploadInput extends _react.Component {
  constructor(props) {
    super(props);
    this.handleTypeInputCommit = this.handleTypeInputCommit.bind(this);
    this.handleFileInputCommit = this.handleFileInputCommit.bind(this);
    this.handleUrlInputCommit = this.handleUrlInputCommit.bind(this);
    this.state = {
      type: props.type
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      type: nextProps.type
    });
  }

  handleTypeInputCommit(path, value) {
    const {
      value: prevValue
    } = this.state;
    this.setState({
      type: value
    });
    const {
      onCommit,
      onTypeChanged
    } = this.props;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), null);
    }

    if (onTypeChanged && value !== prevValue) {
      onTypeChanged(value);
    }
  }

  handleFileInputCommit(path, value) {
    const {
      onCommit
    } = this.props;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  handleUrlInputCommit(path, value) {
    const {
      onCommit
    } = this.props;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  renderTypeInput() {
    const {
      readOnly,
      typeInputLabel,
      fileOptionLabel,
      urlOptionLabel
    } = this.props;
    const {
      type
    } = this.state;
    const options = [{
      value: 'file',
      label: fileOptionLabel
    }, {
      value: 'url',
      label: urlOptionLabel
    }];
    return /*#__PURE__*/_react.default.createElement(DropdownMenuInput, {
      label: typeInputLabel,
      name: "type",
      readOnly: readOnly,
      options: options,
      value: type,
      onCommit: this.handleTypeInputCommit
    });
  }

  renderFileInput() {
    const {
      accept,
      readOnly,
      value,
      fileInputLabel,
      fileChooseButtonLabel,
      formatFileInfo
    } = this.props;
    const fileList = value instanceof Array ? value : null;
    return /*#__PURE__*/_react.default.createElement(FileInput, {
      accept: accept,
      label: fileInputLabel,
      name: "file",
      readOnly: readOnly,
      value: fileList,
      chooseButtonLabel: fileChooseButtonLabel,
      formatFileInfo: formatFileInfo,
      onCommit: this.handleFileInputCommit
    });
  }

  renderUrlInput() {
    const {
      readOnly,
      value,
      urlInputLabel
    } = this.props;
    const externalUrl = typeof value === 'string' ? value : null;
    return /*#__PURE__*/_react.default.createElement(TextInput, {
      label: urlInputLabel,
      name: "url",
      readOnly: readOnly,
      value: externalUrl,
      onCommit: this.handleUrlInputCommit
    });
  }

  render() {
    const {
      type
    } = this.state;
    const fileOrUrlInput = type === 'file' ? this.renderFileInput() : this.renderUrlInput();
    return /*#__PURE__*/_react.default.createElement("div", {
      className: _UploadInput.default.common
    }, this.renderTypeInput(), fileOrUrlInput);
  }

}

exports.default = UploadInput;
UploadInput.propTypes = propTypes;
UploadInput.defaultProps = defaultProps;