"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = VocabularyInput;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _SubstringFilteringDropdownMenuInput = _interopRequireDefault(require("./SubstringFilteringDropdownMenuInput"));

var _withLabeledOptions = _interopRequireDefault(require("../enhancers/withLabeledOptions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions.default)(_SubstringFilteringDropdownMenuInput.default);
const propTypes = {
  formatVocabularyLabel: _propTypes.default.func,
  recordTypes: _propTypes.default.objectOf(_propTypes.default.object),
  recordType: _propTypes.default.string,
  rootVocabulary: _propTypes.default.string,
  value: _propTypes.default.string
};
const defaultProps = {
  formatVocabularyLabel: (name, config) => {
    const {
      messages
    } = config;

    if (messages) {
      return messages.name.defaultMessage;
    }

    return name;
  },
  recordTypes: {},
  recordType: undefined,
  rootVocabulary: 'all',
  value: undefined
};

function VocabularyInput(props) {
  const {
    formatVocabularyLabel,
    recordTypes,
    recordType,
    rootVocabulary,
    value: vocabularyValue,
    ...remainingProps
  } = props;

  if (!recordType || !recordTypes[recordType] || !recordTypes[recordType].vocabularies) {
    return null;
  }

  const {
    vocabularies
  } = recordTypes[recordType];
  const options = [];

  if (vocabularies[rootVocabulary]) {
    options.push({
      value: rootVocabulary,
      label: formatVocabularyLabel(rootVocabulary, vocabularies[rootVocabulary])
    });
  }

  const vocabularyNames = Object.keys(vocabularies).filter(vocabularyName => vocabularyName !== rootVocabulary && !vocabularies[vocabularyName].disabled).sort((nameA, nameB) => {
    const configA = vocabularies[nameA];
    const configB = vocabularies[nameB]; // Primary sort by sortOrder

    let sortOrderA = configA.sortOrder;
    let sortOrderB = configB.sortOrder;

    if (typeof sortOrderA !== 'number') {
      sortOrderA = Number.MAX_VALUE;
    }

    if (typeof sortOrderB !== 'number') {
      sortOrderB = Number.MAX_VALUE;
    }

    if (sortOrderA !== sortOrderB) {
      return sortOrderA > sortOrderB ? 1 : -1;
    } // Secondary sort by label


    const labelA = formatVocabularyLabel(nameA, configA);
    const labelB = formatVocabularyLabel(nameB, configB); // FIXME: This should be locale aware

    return labelA.localeCompare(labelB);
  });
  vocabularyNames.forEach(vocabularyName => {
    options.push({
      indent: 1,
      value: vocabularyName,
      label: formatVocabularyLabel(vocabularyName, vocabularies[vocabularyName])
    });
  });
  let value = vocabularyValue;

  if (!value) {
    for (let i = 0; i < vocabularyNames.length; i += 1) {
      const vocabularyName = vocabularyNames[i];
      const vocabulary = vocabularies[vocabularyName];

      if (vocabulary.defaultForSearch) {
        value = vocabularyName;
        break;
      }
    }

    if (!value && options.length > 0) {
      value = options[0].value;
    }
  }

  return /*#__PURE__*/_react.default.createElement(SubstringFilteringDropdownMenuInput, _extends({
    blankable: false,
    options: options,
    value: value // Too many props to pass explicitly.
    // eslint-disable-next-line react/jsx-props-no-spreading

  }, remainingProps));
}

VocabularyInput.propTypes = propTypes;
VocabularyInput.defaultProps = defaultProps;