"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = changeable;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * An adapter for onChange events. Returns an enhanced component that handles onChange events fired
 * by the base component, and executes a callback with the new value.
 * @param {string|function} BaseComponent - The component to enhance. This component must accept an
 * onChange prop.
 * @returns {function} The enhanced component, which accepts an onChange prop containing a function
 * to be called with the new value.
 */
function changeable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    autoSyncValue: _propTypes.default.bool,
    onChange: _propTypes.default.func,
    value: _propTypes.default.string
  };
  const defaultProps = {
    autoSyncValue: true,
    onChange: undefined,
    value: ''
  };

  class Changeable extends _react.Component {
    constructor(props) {
      super(props);
      this.state = {
        value: props.value
      };
      this.handleChange = this.handleChange.bind(this);
    }

    componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }

    handleChange(event) {
      const {
        value
      } = event.target;
      const {
        onChange,
        autoSyncValue
      } = this.props;

      if (autoSyncValue) {
        this.setState({
          value
        });
      }

      if (onChange) {
        onChange(value);
      }
    }

    render() {
      const {
        autoSyncValue,
        onChange,
        ...remainingProps
      } = this.props;
      const {
        value
      } = this.state;
      return /*#__PURE__*/_react.default.createElement(BaseComponent // eslint-disable-next-line react/jsx-props-no-spreading
      , _extends({}, remainingProps, {
        value: value,
        onChange: this.handleChange
      }));
    }

  }

  Changeable.propTypes = propTypes;
  Changeable.defaultProps = defaultProps;
  Changeable.displayName = `changeable(${baseComponentName})`;
  return Changeable;
}