"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = repeatable;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _RepeatingInput = _interopRequireDefault(require("../components/RepeatingInput"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Makes an input component possibly repeating. Returns an enhanced component that accepts a
 * repeating prop. If true, the base component is wrapped in a RepeatingInput; otherwise, the
 * base component is returned unchanged.
 * @param {string|function} BaseComponent - The component to enhance.
 * @returns {function} The enhanced component.
 */
function repeatable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ..._RepeatingInput.default.propTypes,
    repeating: _propTypes.default.bool
  };
  const defaultProps = {
    repeating: false
  };

  function Repeatable(props) {
    const {
      repeating,
      reorderable,
      renderOrderIndicator,
      disableRemoveButton,
      onAddInstance,
      onMoveInstance,
      onRemoveInstance,
      ...remainingProps
    } = props;

    if (!repeating) {
      return (
        /*#__PURE__*/
        // eslint-disable-next-line react/jsx-props-no-spreading
        _react.default.createElement(BaseComponent, remainingProps)
      );
    }

    const {
      name,
      parentPath,
      subpath,
      value,
      asText,
      readOnly,
      onCommit,
      ...baseProps
    } = remainingProps;
    return /*#__PURE__*/_react.default.createElement(_RepeatingInput.default, {
      name: name,
      parentPath: parentPath,
      subpath: subpath,
      value: value,
      asText: asText,
      readOnly: readOnly,
      reorderable: reorderable,
      disableRemoveButton: disableRemoveButton,
      renderOrderIndicator: renderOrderIndicator,
      onAddInstance: onAddInstance,
      onCommit: onCommit,
      onMoveInstance: onMoveInstance,
      onRemoveInstance: onRemoveInstance
    }, /*#__PURE__*/_react.default.createElement(BaseComponent, baseProps));
  }

  Repeatable.propTypes = propTypes;
  Repeatable.defaultProps = defaultProps;
  Repeatable.displayName = `repeatable(${baseComponentName})`;
  return Repeatable;
}