"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = withLabeledOptions;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * Sets labels on options using a formatting function before passing them on to the base
 * component.
 * @param {string|function} BaseComponent - The component to enhance.
 * @returns {function} The enhanced component.
 */
function withLabeledOptions(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    formatOptionLabel: _propTypes.default.func,
    options: _propTypes.default.arrayOf(_propTypes.default.shape({
      value: _propTypes.default.string
    }))
  };
  const defaultProps = {
    formatOptionLabel: option => typeof option.label === 'undefined' ? option.value : option.label,
    options: []
  };

  function WithLabeledOptions(props) {
    const {
      formatOptionLabel,
      options,
      ...remainingProps
    } = props;
    const labeledOptions = options.map(option => ({ ...option,
      label: formatOptionLabel(option)
    }));
    return /*#__PURE__*/_react.default.createElement(BaseComponent // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      options: labeledOptions
    }));
  }

  WithLabeledOptions.propTypes = propTypes;
  WithLabeledOptions.defaultProps = defaultProps;
  WithLabeledOptions.displayName = `withLabeledOptions(${baseComponentName})`;
  return WithLabeledOptions;
}