"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = withNormalizedOptions;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _optionHelpers = require("../helpers/optionHelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/**
 * Normalizes options before passing them on to the base component.
 * @param {string|function} BaseComponent - The component to enhance.
 * @returns {function} The enhanced component.
 */
function withNormalizedOptions(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
  const propTypes = { // TODO: Stop using propTypes in isInput, and in render method of cspace-ui Field component.
    // Until then, propTypes need to be hoisted from the base component.
    // eslint-disable-next-line react/forbid-foreign-prop-types
    ...BaseComponent.propTypes,
    blankable: _propTypes.default.bool,
    prefilter: _propTypes.default.func,
    options: _propTypes.default.arrayOf(_propTypes.default.shape({
      value: _propTypes.default.string
    })),
    sortComparator: _propTypes.default.func
  };
  const defaultProps = {
    blankable: true,
    options: undefined,
    prefilter: undefined,
    sortComparator: undefined
  };

  function WithNormalizedOptions(props) {
    const {
      blankable,
      options,
      prefilter,
      sortComparator,
      ...remainingProps
    } = props;
    let normalizedOptions = (0, _optionHelpers.normalizeOptions)(options, blankable);

    if (prefilter) {
      normalizedOptions = normalizedOptions.filter(prefilter);
    }

    if (sortComparator) {
      normalizedOptions.sort(sortComparator);
    }

    return /*#__PURE__*/_react.default.createElement(BaseComponent // eslint-disable-next-line react/jsx-props-no-spreading
    , _extends({}, remainingProps, {
      options: normalizedOptions
    }));
  }

  WithNormalizedOptions.propTypes = propTypes;
  WithNormalizedOptions.defaultProps = defaultProps;
  WithNormalizedOptions.displayName = `withNormalizedOptions(${baseComponentName})`;
  return WithNormalizedOptions;
}